#include "resource.h"
#include "zor.h"

using namespace std;

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//@@  CONSTANTS
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
const UI32 NB_ERASE_PASS_STRONG	= 35; // number of times the file is erased
const UI32 NB_ERASE_PASS_MEDIUM	= 18; // number of times the file is erased
const UI32 NB_ERASE_PASS_LOW	= 1; // number of times the file is erased
const UI32 UNIT_SIZE			= 256000;
const SI32 IDT_TIMER1			= 1974;

char zeroed_buf[UNIT_SIZE];


//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//@@  GLOBAL VARIABLE
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Cglobal glob;
HWND remote_hwnd;
unsigned long idc_status	= IDC_STATUS;
unsigned long idc_progress1	= IDC_PROGRESS1;
unsigned long idc_progress2	= IDC_PROGRESS2;


Cglobal::Cglobal () {
	recursive				= false;
	glob.nb_erase_pass			= NB_ERASE_PASS_STRONG;
	folder_scan_cancelled	= false;
	running_flag			= false;
}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;
	LoadString(hInstance, IDS_APP_TITLE, glob.szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_ZOR, glob.szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	if (!InitInstance (hInstance, nCmdShow))  {
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_ZOR);
	while (GetMessage(&msg, NULL, 0, 0))  {
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))  {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	return msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance) {
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_ZOR);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_ZOR;
	wcex.lpszClassName	= glob.szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_LOCK);
	return RegisterClassEx(&wcex);
}

string open_file () {
   OPENFILENAME ofn;
    char szFileName[MAX_PATH] = "";
    ZeroMemory(&ofn, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner	= glob.saved_hWnd;
	ofn.lpstrFilter = "All Files (*.*)\0*.*\0";
    ofn.lpstrFile	= szFileName;
    ofn.nMaxFile	= MAX_PATH;
    ofn.Flags		= OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
    ofn.lpstrDefExt = "txt";
    GetOpenFileName(&ofn);
	return(szFileName);	
}

//=============================================================
//=============================================================
bool file_exists (const string& _filename) {
	bool bret = true;

	FILE * fp = fopen (_filename.c_str(), "rb");
	if (!fp) {
		bret = false;
	}
	else {
		fclose(fp);
	}	

	return(bret);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;
	glob.hInst = hInstance; // Store instance handle in our global variable

	hWnd = CreateWindow(glob.szWindowClass, glob.szTitle, WS_VISIBLE | WS_OVERLAPPEDWINDOW,
	  CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

	if (!hWnd) {
	  return FALSE;
	}

	ShowWindow(hWnd, SW_SHOWMINIMIZED);
	UpdateWindow(hWnd);
	glob.saved_hWnd = hWnd;

	DialogBox(glob.hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
	DestroyWindow(hWnd);
	return TRUE;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
	
	TCHAR szHello[MAX_LOADSTRING];
	LoadString(glob.hInst, IDS_HELLO, szHello, MAX_LOADSTRING);

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_ABOUT:
				   DialogBox(glob.hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   break;
				case IDM_EXIT:
				   DestroyWindow(hWnd);
				   break;

				case IDM_OPEN:
					open_file();
					break;

				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			glob.glob_hdc = hdc;
			// TODO: Add any drawing code here...
			RECT rt;
			GetClientRect(hWnd, &rt);
			DrawText(hdc, szHello, strlen(szHello), &rt, DT_CENTER);
			EndPaint(hWnd, &ps);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

//====================================================================
// just for a pretty display
//====================================================================
void center_window (HWND _hDlg) {
	if (_hDlg) {
		RECT rect; 
		GetWindowRect (_hDlg,&rect); 
		int x = (GetSystemMetrics(SM_CXSCREEN)-rect.right)/2;
		int y = (GetSystemMetrics(SM_CYSCREEN)-rect.bottom)/2;
		SetWindowPos (_hDlg, HWND_TOP, x, y, 0, 0, SWP_NOSIZE);
	}
	else {
		MessageBox(glob.saved_hWnd, "cant center window", "",MB_OK);
	}	
}



//-------------------------------------------------------------
string choose_directory () {
	OleInitialize (NULL);
	CoInitialize (NULL);

	string result = "";
	BROWSEINFO bi;
	memset(&bi,0,sizeof(BROWSEINFO));
	char buffer[MAX_PATH];
	bi.hwndOwner=glob.saved_hWnd;
	// Contient le repertoire initial ou NULL
	bi.pidlRoot=NULL;
	bi.pszDisplayName=buffer;
	bi.lpszTitle="Current directory";
	bi.ulFlags= BIF_NEWDIALOGSTYLE;
	bi.lParam=NULL;

	LPITEMIDLIST li = SHBrowseForFolder(&bi);

	if (li) {
		SHGetPathFromIDList(li, buffer);
		result = buffer;
	}
	return (result);
}

//============================================================
bool erase_ktimes_content (CCPTR _filename) {
	bool bret = true;
	Cbar file_bar;

	// extract file size
	UI32 file_size = 0;
	bret = get_file_size (_filename, file_size);
	if (bret == true) {
		UI32 nb_unit		= file_size / UNIT_SIZE;
		UI32 real_nb_unit	= nb_unit;
		UI32 nb_remaining	= file_size - (nb_unit * UNIT_SIZE); 
		if (nb_remaining > 0) {
			real_nb_unit++;	
		}

		file_bar.init (glob.saved_hWnd, IDC_PROGRESS1, real_nb_unit * glob.nb_erase_pass);
		char tmp[256];

		for (glob.current_pass=0 ; glob.current_pass<glob.nb_erase_pass ; glob.current_pass++) {
			snprintf (tmp, 256, "erasing content (pass %u)", glob.current_pass+1);
			tmp[256 - 1] = '\0';
			SendDlgItemMessage (glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) tmp);
			bret = erase_content_fopen (_filename, file_size, file_bar);
			if (bret == false) {
				break;
			}
			if (glob.folder_scan_cancelled == true) {
				break;
			}	
		}
	}
	return (bret);
}

void generate_random_buffer (char * _buf, UI32 _size) {
	for (UI32 z=0 ; z<_size ; z++) {
		//_buf[z] = rand ();
		_buf[z] = glob.rc4_gen.get_next_byte ();
	}
}

void reset_random_buffer (char * _buf, UI32 _size) {
	for (UI32 z=0 ; z<_size ; z++) {
		_buf[z] = 0x00;
	}
}

bool erase_content_fopen (CCPTR _filename, UI32 _file_size, Cbar& file_bar) {
	bool bret = true;
	UI32 size = _file_size;
	FILE * fp = fopen (_filename, "wb");
	if (! fp) {
		char * str = strerror (errno);
		MessageBox (NULL, str, _filename, MB_ICONERROR);
		bret = false;
	}
	else {
		UI32 nb_unit		= size / UNIT_SIZE;
		UI32 real_nb_unit	= nb_unit;
		UI32 nb_remaining	= size - (nb_unit * UNIT_SIZE); 
		if (nb_remaining > 0) {
			real_nb_unit++;	
		}

		fseek (fp, 0, SEEK_SET);

		if (zeroed_buf == NULL) {
			bret = false;
		}
		else {
			generate_random_buffer (zeroed_buf, UNIT_SIZE);
			UI32 nb_done = 0;

			for (UI32 i=0 ; i<nb_unit ; i++) {
				generate_random_buffer (zeroed_buf, UNIT_SIZE); 

				if (fwrite (zeroed_buf, 1, UNIT_SIZE, fp) != UNIT_SIZE) {
					bret = false;
					break;
				}
				file_bar.incr (1);
				nb_done += UNIT_SIZE;
				// cancel when erasing file is too dangerous !
				/*
				if (glob.folder_scan_cancelled == true) {
					break;
				}*/
			}

			if (glob.folder_scan_cancelled == false) {
				if ((bret == true) && (nb_remaining > 0)) {
					generate_random_buffer (zeroed_buf, UNIT_SIZE);
					if (fwrite (zeroed_buf, 1, nb_remaining, fp) != nb_remaining) {
						bret = false;
					}
					file_bar.incr (1);
				}
			}

			if (fclose (fp) != 0) {
				bret = false;
			}
		}
	}
	reset_random_buffer (zeroed_buf, UNIT_SIZE);
	return (bret);
}

bool doit_encode_file (CCPTR _filename) {
	bool bret = true;
	bool silent_mode = false;

	string chosen_file;
	if (_filename == NULL) {
		chosen_file = open_file ();
	}
	else {
		chosen_file = _filename;
		silent_mode	= true;
	}
	bool content_erased = false;
	bool mmap_failed	= false;

	if (chosen_file != "") {
		remove_system_attribute (chosen_file.c_str());		// ok if it fails
		remove_hidden_attribute (chosen_file.c_str());		// ok if it fails
		remove_read_only_protection (chosen_file.c_str());	// ok if it fails

		Cfile_mapper mapper;
		UI08 * buf_ptr = NULL;
		UI32 size;
		if (silent_mode == false) {
			SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "erasing content...");
		}
		if (1==1) {
		//if (mapper.mmap_file (chosen_file.c_str(), MMAP_RW, buf_ptr, size) == false) {
			/*
			UI32 ercode, ersource;
			mapper.get_last_mmap_error (ercode, ersource);
			if (silent_mode == false) {
				snprintf (tmp, MAX_PATH, "failed to encode file (mmap failed - ercode=%u,ersource=%u)\nfile: %s", ercode, ersource, chosen_file.c_str());
				tmp[MAX_PATH - 1] = '\0';
				MessageBox (NULL, tmp, "Encode Error", MB_ICONERROR);
			}*/
			mmap_failed = true;
		}
		else {
			Cbar file_bar;
			UI32 nb_unit		= size / UNIT_SIZE;
			UI32 real_nb_unit	= nb_unit;
			UI32 nb_remaining	= size - (nb_unit * UNIT_SIZE); 
			if (nb_remaining > 0) {
				real_nb_unit++;	
			}/*
			{
				char tmp[MAX_PATH];
				snprintf (tmp, MAX_PATH, "nb_unit=%u|nb_remaining=%u|real_nb_unit=%u", nb_unit, nb_remaining, real_nb_unit);
				tmp[MAX_PATH - 1] = '\0';
				MessageBox (NULL, tmp, "", MB_OK);
			}*/
			file_bar.init (glob.saved_hWnd, IDC_PROGRESS1, real_nb_unit);
			UI32 nb_done = 0;

			for (UI32 i=0 ; i<nb_unit ; i++) {
				memset (&buf_ptr[nb_done], 0x00, UNIT_SIZE);
				file_bar.incr (1);
				nb_done += UNIT_SIZE;
			}

			if (nb_remaining > 0) {
				memset (&buf_ptr[nb_done], 0x00, nb_remaining);
				file_bar.incr (1);
			}
			mapper.close_mmaped_file ();
			content_erased = true;
		}
 
		if (mmap_failed == true) { // attempt with fopen
			bret = erase_ktimes_content (chosen_file.c_str());
			if (bret == true) {
				content_erased = true;	
			}
			else {
				char tmp[MAX_PATH];
				snprintf (tmp, MAX_PATH, "failed to erase file content :\n%s", chosen_file.c_str());
				tmp[MAX_PATH - 1] = '\0';
				MessageBox (NULL, tmp, "", MB_ICONERROR);
			}
		}


		if ((glob.folder_scan_cancelled == false) && (content_erased == true)) {
			//MessageBox (NULL, "step 1 - content erased...[CHECK]", "", MB_OK);
			string new_filename = extract_folder_from_filename (chosen_file.c_str());
			new_filename = new_filename + "a";
			//MessageBox (NULL, new_filename.c_str(), "", MB_OK);
			if (silent_mode == false) {
				SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "moving file...");
			}
			bret = move_file (chosen_file.c_str(), new_filename.c_str());
			if (bret == false) {
				char tmp[MAX_PATH];
				snprintf (tmp, MAX_PATH, "Failed to move file :\n%s -> %s", chosen_file.c_str(), new_filename.c_str());
				tmp[MAX_PATH - 1] = '\0';
				MessageBox (NULL, tmp, "", MB_ICONERROR);
			}
			else {
				//MessageBox (NULL, "step 2 - file moved...[CHECK]", "", MB_OK);
				SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "deleting file...");
				if (DeleteFile (new_filename.c_str()) == 0) {
					char tmp[MAX_PATH];
					snprintf (tmp, MAX_PATH, "Failed to delete file :\n%s", new_filename.c_str());
					tmp[MAX_PATH - 1] = '\0';
					MessageBox (NULL, tmp, "", MB_ICONERROR);
				}
				else {
					//MessageBox (NULL, "step 3 - file erased...[CHECK]", "", MB_OK);
				}
			}
		}
	}

	if (silent_mode == false) {
		if (glob.folder_scan_cancelled == true) {
			SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Operation cancelled !");
			Cbar file_bar, thread_bar; // reset all progress bars
			file_bar.init	(glob.saved_hWnd, IDC_PROGRESS1, NB_BOOTLEG);
			thread_bar.init (glob.saved_hWnd, IDC_PROGRESS2, NB_BOOTLEG);
			HWND hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FILE); 
			SetWindowText (hwnd_std, "Erase File");
		}
		else if (bret == true) {
			SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Erased succesfully");
		}
		else {
			SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Erase Failed !");
		}
	}
	return (bret);
}

DWORD WINAPI doit_encode_file_threaded (LPVOID lParam) {
	char * filename = (char *) lParam;
	HWND hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER);	
	EnableWindow (hwnd_std, FALSE);
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FILE); 
	SetWindowText (hwnd_std, "Stop");
	glob.running_flag = true;

	SI32 iret = doit_encode_file (filename);

	if (glob.folder_scan_cancelled == true) {
		MessageBox (NULL, "Operation Cancelled !", "", MB_ICONWARNING);
		glob.folder_scan_cancelled = false;
	}

	glob.running_flag = false;
	reset_view ();
	enable_controls (true, true);
	return (iret);
}

bool gather_files_and_folders_not_recursive (CCPTR _chosen_folder) {
	bool bret = true;
	char old_dir[MAX_PATH];
	GetCurrentDirectory (MAX_PATH, old_dir);
	if (SetCurrentDirectory (_chosen_folder) == false) {
		char tmp[MAX_PATH];
		snprintf (tmp, MAX_PATH, "SetCurrentDirectory failed :\n%s", _chosen_folder);
		tmp[MAX_PATH - 1] = '\0';
		MessageBox (NULL, tmp, "", MB_ICONERROR);
		bret = false;
	}
	else {
		SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "creating list...");
		WIN32_FIND_DATA FindData; 
		HANDLE hFind = FindFirstFile ("*", &FindData);

		if (hFind == INVALID_HANDLE_VALUE) { // no file found at all
			bret = false;
		}
		else {
			string	full_filename;
			do { // if it is not a directory
				if (!(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
					full_filename = _chosen_folder;
					full_filename = full_filename + "\\" + FindData.cFileName;
					glob.v_file.push_back (full_filename);
				}

				if (glob.folder_scan_cancelled == true) {
					break;
				}
			}
			while (FindNextFile (hFind, &FindData)) ;
		}
		FindClose (hFind);
	}
	SetCurrentDirectory (old_dir);
	return (bret);
}

bool doit_encode_folder () {
	bool bret = true;
	UI32 nb_done  = 0;
	UI32 nb_error = 0;
	char tmp[MAX_PATH];
	Cbar file_bar;
	UI32 nb_elem = glob.v_file.size ();

	file_bar.init (glob.saved_hWnd, IDC_PROGRESS2, nb_elem);
	for (UI32 i=0 ; i<nb_elem ; i++) {
		bret = doit_encode_file (glob.v_file[i].c_str());
		file_bar.incr ();
		if (bret == false) {
			nb_error++;
		}
		nb_done++;
		snprintf (tmp, MAX_PATH, "%u done / %u failed", nb_done, nb_error);
		tmp[MAX_PATH - 1] = '\0';
		SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) tmp);
		if (glob.folder_scan_cancelled == true) {
			break;
		}
	}

	if (nb_error > 0) {
		bret = false;
		MessageBox (NULL, "Operation Failed !", "", MB_ICONERROR);
	}
	else {
		if (nb_done == 0) {
			SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "no file found !");
			MessageBox (NULL, "no file found !", "", MB_ICONWARNING);
		}
		else if (glob.folder_scan_cancelled == false) {
			MessageBox (NULL, "Operation finished", "", MB_OK);
		}
	}
	return (bret);
}

bool doit_erase_folders () {
	bool bret = true;
	sort (glob.v_folder.begin(), glob.v_folder.end());
	UI32 nb_folder = glob.v_folder.size ();
	for (UI32 i=0 ; i<nb_folder ; i++) {
		if (RemoveDirectory (glob.v_folder[i].m_name.c_str()) == 0) {
			DWORD ercode = GetLastError ();
			bret = false;
		}

		if (glob.folder_scan_cancelled == true) {
			break;
		}
	}
	return (bret);
}

DWORD WINAPI doit_process_folder (LPVOID lParam) {
	string chosen_folder = choose_directory ();
	if (chosen_folder != "") {
		glob.running_flag = true;
		HWND hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER); 
		SetWindowText (hwnd_std, "Stop");
		disable_controls (true, false);

		glob.v_file.clear ();
		glob.v_folder.clear ();
		char orig_folder[FOLDER_NAME_SIZE];
		GetCurrentDirectory (FOLDER_NAME_SIZE, orig_folder);

		if (glob.recursive == true) {
			gather_files_and_folders_recursive (chosen_folder.c_str());
		}
		else {
			gather_files_and_folders_not_recursive (chosen_folder.c_str());
		}

		if (glob.folder_scan_cancelled == false) {
			SetCurrentDirectory (orig_folder);
			doit_encode_folder ();

			if (glob.folder_scan_cancelled == false) {
				if (glob.recursive == true) {
					bool bret = doit_erase_folders ();
					if (bret == false) {
						MessageBox (NULL, "One or more folders could not be erased !", "", MB_ICONERROR);
					}
				}
			}
		}
	}

	if (glob.folder_scan_cancelled == true) {
		MessageBox (NULL, "Operation Cancelled !", "", MB_ICONWARNING);
		HWND hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER); 
		SetWindowText (hwnd_std, "Erase Folder");
		glob.folder_scan_cancelled = false;
	}
	glob.running_flag = false;
	reset_view ();
	enable_controls (true, true);
	return (0);
}

bool create_folder_encoding_thread () {
	bool bret = true;
	DWORD dwThreadId;
	HANDLE hThread; 
	hThread = CreateThread (NULL,0, doit_process_folder, NULL,0,&dwThreadId);
	if (hThread == NULL)   {
		MessageBox (NULL, "Thread creation failed", "Error", MB_OK | MB_ICONERROR);			
		bret = false;
	}
	return (bret);
}

bool create_file_encoding_thread (CCPTR _filename) {
	bool bret = true;
	DWORD dwThreadId;
	HANDLE hThread; 
	hThread = CreateThread (NULL,0, doit_encode_file_threaded, (LPVOID)_filename,0,&dwThreadId);
	if (hThread == NULL)   {
		MessageBox (NULL, "Thread creation failed", "Error", MB_OK | MB_ICONERROR);			
		bret = false;
	}
	return (bret);
}

//--------------------------------------------------
bool extract_directory_list (string& dirname, vector<string>& v_dir) {
	bool bret = true;

	if (SetCurrentDirectory (dirname.c_str()) == false) {
		bret = false;
	}
	else {
		WIN32_FIND_DATA FindData; 
		HANDLE hFind = FindFirstFile ("*.*", &FindData);	

		if (hFind == INVALID_HANDLE_VALUE) {
			bret = false;
		}
		else {
			v_dir.clear ();
			int name_size;

			do  {
				if (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
					if((strcmp(FindData.cFileName, ".")!=0) && (strcmp(FindData.cFileName, "..")!=0)) {
						name_size = dirname.size ();
						if (name_size > 0) {
							if (dirname[name_size - 1] != '\\') {
								v_dir.push_back (dirname + "\\" + FindData.cFileName);
							}
							else {
								v_dir.push_back (dirname + FindData.cFileName);
							}
						}
					}
				}

				if (glob.folder_scan_cancelled == true) {
					break;
				}
			}
			while (FindNextFile (hFind, &FindData)) ;
		}
		FindClose (hFind);
	}
	return (bret);
}

bool explore_one_directory (string dirname, string pattern_arg) {
	bool bret = true;
	if (SetCurrentDirectory (dirname.c_str()) == false) {
		bret = false;
	}
	else {
		WIN32_FIND_DATA FindData; 
		HANDLE hFind = FindFirstFile (pattern_arg.c_str(), &FindData);

		if (hFind == INVALID_HANDLE_VALUE) { // no file found at all
			bret = true;
		}
		else {
			string full_filename = "";
			do {
				// if it is not a directory
				if (!(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
					 full_filename = dirname + "\\" + FindData.cFileName;
					 glob.v_file.push_back (full_filename.c_str());
				}
				else if ((strcmp(FindData.cFileName, ".")!=0) && (strcmp(FindData.cFileName, "..")!=0)) {
					Cfolder_unit unit;
					unit.m_name = dirname + "\\" + FindData.cFileName;
					glob.v_folder.push_back (unit);
				}
				if (glob.folder_scan_cancelled == true) {
					break;
				}
			}
			while (FindNextFile (hFind, &FindData)) ;
			FindClose (hFind);
		}
	}
	return (bret);
}

void explore_all_from_local_root (string root_arg) {
	bool bret;
	vector<string>	v_dir;
	bret = explore_one_directory (root_arg, "*");

	if (glob.folder_scan_cancelled == false) {
		extract_directory_list (root_arg, v_dir);

		if (glob.folder_scan_cancelled == false) {
			SI32 size = v_dir.size ();
			for (SI32 i=0 ; i<size ; i++) {
				explore_all_from_local_root (v_dir[i]);
				if (glob.folder_scan_cancelled == true) {
					break;
				}
			}
		}
	}
}

// folders are sorted so that the deepest are the first of the container
void gather_files_and_folders_recursive (CCPTR _folder_2_process) {
	SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Creating list...");
	glob.folder_scan_cancelled = false;

	explore_all_from_local_root (_folder_2_process);
	UI32 nb_file = glob.v_file.size ();
	SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "File List created");
}

//=============================================================

void prepare_combo () {
	SendDlgItemMessage (glob.saved_hWnd, IDC_STRENGTH, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)"Low");
	SendDlgItemMessage (glob.saved_hWnd, IDC_STRENGTH, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)"Medium");
	SendDlgItemMessage (glob.saved_hWnd, IDC_STRENGTH, CB_ADDSTRING, 0, (LPARAM)(LPCSTR)"High");
}

void disable_controls (bool _is_folder, bool _is_file) {
	HWND hwnd_std;
	if (_is_folder == true) {
		hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FILE);	EnableWindow (hwnd_std, FALSE);
	}
	if (_is_file == true) {
		hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER);	EnableWindow (hwnd_std, FALSE);
	}
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_STRENGTH);		EnableWindow (hwnd_std, FALSE);
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_RECUR);			EnableWindow (hwnd_std, FALSE);
}

void enable_controls (bool _is_folder, bool _is_file) {
	HWND hwnd_std;
	if (_is_folder == true) {
		hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FILE);	EnableWindow (hwnd_std, TRUE);
	}
	if (_is_file == true) {
		hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER);	EnableWindow (hwnd_std, TRUE);
	}
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_STRENGTH);		EnableWindow (hwnd_std, TRUE);
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_RECUR);			EnableWindow (hwnd_std, TRUE);
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FILE); 
	SetWindowText (hwnd_std, "Erase File");
	hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER); 
	SetWindowText (hwnd_std, "Erase Folder");
}

bool acquire_nb_pass () {
	bool bret = true;
	glob.nb_erase_pass = NB_ERASE_PASS_STRONG; 

	int cell_index = SendDlgItemMessage (glob.saved_hWnd, IDC_STRENGTH, CB_GETCURSEL, 0, 0);
	char tmp[256];
	SendDlgItemMessage (glob.saved_hWnd, IDC_STRENGTH, CB_GETLBTEXT, (WORD)cell_index, (LONG)tmp);

	if (strcmp (tmp, "High") == 0) {
		glob.nb_erase_pass = NB_ERASE_PASS_STRONG;
		SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Ready (strong entropy: 35 pass)");
	}
	else if (strcmp (tmp, "Medium") == 0) {
		glob.nb_erase_pass = NB_ERASE_PASS_MEDIUM;
		SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Ready (medium entropy: 18 pass)");
	}
	else if (strcmp (tmp, "Low") == 0) {
		SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Ready (Low entropy: 1 pass)");
		glob.nb_erase_pass = NB_ERASE_PASS_LOW;
	}
	else {
		MessageBox (NULL, "Please choose an entropy strength !", "Strength not chosen", MB_ICONERROR);
		bret = false;
	}
	return (bret);
}

void reset_view () {
	Cbar file_bar;
	file_bar.init (glob.saved_hWnd, IDC_PROGRESS1, 10);	
	file_bar.init (glob.saved_hWnd, IDC_PROGRESS2, 10);
	acquire_nb_pass ();
}

DWORD WINAPI doit_prepare_random_generator (LPVOID lParam) {
	SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "preparing random generator...");
	glob.rc4_gen.prepare_using_random_key ();
	SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Ready");
	return (0);
}

void prepare_random_generator () {
	bool bret = true;
	DWORD dwThreadId;
	HANDLE hThread; 
	hThread = CreateThread (NULL,0, doit_prepare_random_generator, NULL,0,&dwThreadId);
	if (hThread == NULL)   {
		MessageBox (NULL, "Thread creation failed", "Error", MB_OK | MB_ICONERROR);			
		bret = false;
	}
}

void display_cancelled_operation () {
	char tmp[TMP_BUFSIZE];
	snprintf (tmp, TMP_BUFSIZE, "<CANCEL SCHEDULED> erasing content (pass %u)", glob.current_pass+1);
	tmp[TMP_BUFSIZE - 1] = '\0';
	SendDlgItemMessage (glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) tmp);
	disable_controls (true, true);
}

//-------------------------------------------------------------
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			glob.saved_hWnd	= hDlg;
			remote_hwnd = hDlg;
			center_window (hDlg);
			prepare_combo ();
			SetTimer (hDlg, IDT_TIMER1, 2000, (TIMERPROC) NULL);
			SendDlgItemMessage(glob.saved_hWnd, IDC_STATUS, WM_SETTEXT, 0, (LPARAM) "Preparing...(please wait)");
			return TRUE;

		case WM_TIMER: // this message is sent automatically when timer expires
			if (wParam == IDT_TIMER1) { // the identifier of the timer that has fired
				KillTimer (hDlg, IDT_TIMER1);
				prepare_random_generator ();
			}
			break;

		case WM_COMMAND:
			switch( HIWORD( wParam ) ) {
				case CBN_SELCHANGE:
					acquire_nb_pass ();
					break;
			}

			switch(LOWORD(wParam)) {
				case IDOK:
				case IDCANCEL:
					ShowWindow (hDlg, SW_SHOW);
					EndDialog(hDlg, LOWORD(wParam));
					return TRUE;
				break;

				case IDC_RECUR:
					if (IsDlgButtonChecked (hDlg, IDC_RECUR) == BST_CHECKED) {
						glob.recursive = true;
					}
					else {
						glob.recursive = false;
					}
					break;
				
				case IDC_ENCODE_FILE:
					if (glob.running_flag == false) {
						if (acquire_nb_pass () == true) {
							create_file_encoding_thread (NULL);
						}
					}
					else if (MessageBox (hDlg, "Are you sure you want to abort operation ?", "Question", MB_YESNO | MB_ICONWARNING)==IDYES) {
						display_cancelled_operation ();
						glob.folder_scan_cancelled = true;
						glob.running_flag = false;
					}
					break;

				case IDC_ENCODE_FOLDER: 
					if (glob.running_flag == false) {
						if (acquire_nb_pass () == true) {
							create_folder_encoding_thread ();
						}
					}
					else if (MessageBox (hDlg, "Are you sure you want to abort operation ?", "Question", MB_YESNO | MB_ICONWARNING)==IDYES) { 
						display_cancelled_operation ();
						glob.folder_scan_cancelled = true;
						glob.running_flag = false;
						HWND hwnd_std = GetDlgItem (glob.saved_hWnd, IDC_ENCODE_FOLDER); 
						SetWindowText (hwnd_std, "Erase Folder");
					}
					break;
			}
	}
	return FALSE;
}
