// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: des_tool.cpp
// @@ 
// @@ DESCRIPTION:
// @@ This file permits to encipher/decipher buffers
// @@ and files using the Data Encryption Standard.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include "des_tool.hpp"



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ This matrix is the identity matrix 
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 identity[] = {
1,2,3,4,5,6,7,8,9,
10,11,12,13,14,15,16,17,
18,19,20,21,22,23,24,25,
26,27,28,29,30,31,32,33,
34,35,36,37,38,39,40,41,
42,43,44,45,46,47,48,49,
50,51,52,53,54,55,56,57,
58,59,60,61,62,63,64
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ initial permutation IP
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 ip[] = {
        58, 50, 42, 34, 26, 18, 10,  2,
        60, 52, 44, 36, 28, 20, 12,  4,
        62, 54, 46, 38, 30, 22, 14,  6,
        64, 56, 48, 40, 32, 24, 16,  8,
        57, 49, 41, 33, 25, 17,  9,  1,
        59, 51, 43, 35, 27, 19, 11,  3,
        61, 53, 45, 37, 29, 21, 13,  5,
        63, 55, 47, 39, 31, 23, 15,  7
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ final permutation IP^-1
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 fp[] = {
        40,  8, 48, 16, 56, 24, 64, 32,
        39,  7, 47, 15, 55, 23, 63, 31,
        38,  6, 46, 14, 54, 22, 62, 30,
        37,  5, 45, 13, 53, 21, 61, 29,
        36,  4, 44, 12, 52, 20, 60, 28,
        35,  3, 43, 11, 51, 19, 59, 27,
        34,  2, 42, 10, 50, 18, 58, 26,
        33,  1, 41,  9, 49, 17, 57, 25
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ first permuted choice table (left part, permitting to generate C0)
// @@ (used during key schedule calculation).
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 pc1_left[] = {
        57, 49, 41, 33, 25, 17,  9,
         1, 58, 50, 42, 34, 26, 18,
        10,  2, 59, 51, 43, 35, 27,
        19, 11,  3, 60, 52, 44, 36,
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ first permuted choice table (right part, permitting to generate D0)
// @@ (used during key schedule calculation).
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 pc1_right[] = {
        63, 55, 47, 39, 31, 23, 15,
         7, 62, 54, 46, 38, 30, 22,
        14,  6, 61, 53, 45, 37, 29,
        21, 13,  5, 28, 20, 12,  4
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ permuted choice key (table)
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 pc2[] = {
        14, 17, 11, 24,  1,  5,
         3, 28, 15,  6, 21, 10,
        23, 19, 12,  4, 26,  8,
        16,  7, 27, 20, 13,  2,
        41, 52, 31, 37, 47, 55,
        30, 40, 51, 45, 33, 48,
        44, 49, 39, 56, 34, 53,
        46, 42, 50, 36, 29, 32
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ expansion operation matrix
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 ei[] = {
        32,  1,  2,  3,  4,  5,
         4,  5,  6,  7,  8,  9,
         8,  9, 10, 11, 12, 13,
        12, 13, 14, 15, 16, 17,
        16, 17, 18, 19, 20, 21,
        20, 21, 22, 23, 24, 25,
        24, 25, 26, 27, 28, 29,
        28, 29, 30, 31, 32,  1
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ The (in)famous S-boxes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 si[] = {
        // S1
        14,  4, 13,  1,  2, 15, 11,  8,  3, 10,  6, 12,  5,  9,  0,  7,
         0, 15,  7,  4, 14,  2, 13,  1, 10,  6, 12, 11,  9,  5,  3,  8,
         4,  1, 14,  8, 13,  6,  2, 11, 15, 12,  9,  7,  3, 10,  5,  0,
        15, 12,  8,  2,  4,  9,  1,  7,  5, 11,  3, 14, 10,  0,  6, 13,

        // S2
        15,  1,  8, 14,  6, 11,  3,  4,  9,  7,  2, 13, 12,  0,  5, 10,
         3, 13,  4,  7, 15,  2,  8, 14, 12,  0,  1, 10,  6,  9, 11,  5,
         0, 14,  7, 11, 10,  4, 13,  1,  5,  8, 12,  6,  9,  3,  2, 15,
        13,  8, 10,  1,  3, 15,  4,  2, 11,  6,  7, 12,  0,  5, 14,  9,

        // S3
        10,  0,  9, 14,  6,  3, 15,  5,  1, 13, 12,  7, 11,  4,  2,  8,
        13,  7,  0,  9,  3,  4,  6, 10,  2,  8,  5, 14, 12, 11, 15,  1,
        13,  6,  4,  9,  8, 15,  3,  0, 11,  1,  2, 12,  5, 10, 14,  7,
         1, 10, 13,  0,  6,  9,  8,  7,  4, 15, 14,  3, 11,  5,  2, 12,

        // S4
         7, 13, 14,  3,  0,  6,  9, 10,  1,  2,  8,  5, 11, 12,  4, 15,
        13,  8, 11,  5,  6, 15,  0,  3,  4,  7,  2, 12,  1, 10, 14,  9,
        10,  6,  9,  0, 12, 11,  7, 13, 15,  1,  3, 14,  5,  2,  8,  4,
         3, 15,  0,  6, 10,  1, 13,  8,  9,  4,  5, 11, 12,  7,  2, 14,

        // S5
         2, 12,  4,  1,  7, 10, 11,  6,  8,  5,  3, 15, 13,  0, 14,  9,
        14, 11,  2, 12,  4,  7, 13,  1,  5,  0, 15, 10,  3,  9,  8,  6,
         4,  2,  1, 11, 10, 13,  7,  8, 15,  9, 12,  5,  6,  3,  0, 14,
        11,  8, 12,  7,  1, 14,  2, 13,  6, 15,  0,  9, 10,  4,  5,  3,

        // S6
        12,  1, 10, 15,  9,  2,  6,  8,  0, 13,  3,  4, 14,  7,  5, 11,
        10, 15,  4,  2,  7, 12,  9,  5,  6,  1, 13, 14,  0, 11,  3,  8,
         9, 14, 15,  5,  2,  8, 12,  3,  7,  0,  4, 10,  1, 13, 11,  6,
         4,  3,  2, 12,  9,  5, 15, 10, 11, 14,  1,  7,  6,  0,  8, 13,

        // S7
         4, 11,  2, 14, 15,  0,  8, 13,  3, 12,  9,  7,  5, 10,  6,  1,
        13,  0, 11,  7,  4,  9,  1, 10, 14,  3,  5, 12,  2, 15,  8,  6,
         1,  4, 11, 13, 12,  3,  7, 14, 10, 15,  6,  8,  0,  5,  9,  2,
         6, 11, 13,  8,  1,  4, 10,  7,  9,  5,  0, 15, 14,  2,  3, 12,

        // S8
        13,  2,  8,  4,  6, 15, 11,  1, 10,  9,  3, 14,  5,  0, 12,  7,
         1, 15, 13,  8, 10,  3,  7,  4, 12,  5,  6, 11,  0, 14,  9,  2,
         7, 11,  4,  1,  9, 12, 14,  2,  0,  6, 10, 13, 15,  3,  5,  8,
         2,  1, 14,  7,  4, 10,  8, 13, 15, 12,  9,  0,  3,  5,  6, 11
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ 32-bit permutation function P used on the output of the S-boxes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI08 p32i[] = {
        16,  7, 20, 21,
        29, 12, 28, 17,
         1, 15, 23, 26,
         5, 18, 31, 10,
         2,  8, 24, 14,
        32, 27,  3,  9,
        19, 13, 30,  6,
        22, 11,  4, 25
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	convert_2_big_endian
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	
// @@ _var: SI32&: the value that is to be converted into big endian.
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to flip the order of the bytes of 
// @@ the integer given as a parameter so that we can switch to
// @@ and from big endian using it.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void convert_2_big_endian (SI32& _var) {
	A32 a32;
	a32.aint = _var;
	A32 converted_a32;
	converted_a32.atab[0] = a32.atab[3];
	converted_a32.atab[1] = a32.atab[2];
	converted_a32.atab[2] = a32.atab[1];
	converted_a32.atab[3] = a32.atab[0];
	_var = converted_a32.aint;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	convert_2_little_endian
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	
// @@ _var: SI32&: the value that is to be converted into big endian.
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to flip the order of the bytes of 
// @@ the integer given as a parameter so that we can switch to
// @@ and from big endian using it.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void convert_2_little_endian (SI32& _var) {
        A32 a32;
        a32.aint = _var;
        A32 converted_a32;
        converted_a32.atab[0] = a32.atab[3];
        converted_a32.atab[1] = a32.atab[2];
        converted_a32.atab[2] = a32.atab[1];
        converted_a32.atab[3] = a32.atab[0];
        _var = converted_a32.aint;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	display_hex_buf	
// @@
// @@ INPUT			:	
// @@ _label: string: the message to be displayed along with buffer.
// @@
// @@ _buf: UI08 *: the buffer to be displayed to standard output.
// @@ 
// @@ _size: int: the size of the buffer to display.
// @@
// @@ _display_bitfield: bool: true if bitfield is to be displayed,
// @@                          false otherwise.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to display to standard error the buffer
// @@ as either hexa format, or binary format.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void display_hex_buf (string _label, UI08 * _buf, int _size, bool _display_bitfield) {
	char tmp[32];
	cerr << _label << " = ";
	Cdes des;
	
        for (int i=0 ; i<_size ; i++) {
                snprintf (tmp, 32, "%02X ", _buf[i]);
                tmp[31] = '\0';
                cerr << tmp;
		if ((i != 0) && ((i%8) == 0)) {
			cerr << endl;
		}	
        }
        cerr << endl;
	if (_display_bitfield) {
	        for (int i=1 ; i<=8*_size ; i++) {
       	         if (des.is_set(_buf, i)) {
       	                 //cerr << "bit "<< i << ") 1" << endl;
				cerr << 1;
       	         }
       	         else {
       	                 //cerr << "bit "<< i << ") 0" << endl;
				cerr << "0";
       	         }
		 if ((i != 0) && ((i%4) == 0)) {
			cerr << " ";
		 }
       		}
        	cerr << endl;
#if defined (EXTRA_DISPLAY_BITFIELD)		
		for (int i=1 ; i<=8*_size ; i++) {
			if (des.is_set(_buf, i)) {
				cerr << i << ") 1" << endl;
			}
			else {
				cerr << i << ") 0" << endl;
			} 
		}
		cerr << endl;
#endif		
	}
	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	Cdes	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ Normal constructor of the 'Cdes' object.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Cdes::Cdes () {
	reset ();
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	reset	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function brings the Cdes object back to it's 
// @@ initial state.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::reset () {
	key     = NULL;
	block   = NULL;
	result  = NULL;	
	current = 0;	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	do_initial_block_permutation	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to perform the initial block permutation.
// @@ Notice that the result of permutation is in the 'result' member.
// @@
// @@ CONTRACT		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::do_initial_block_permutation () {
	do_xx_permutation (block, result, BLOCK_SIZE, ip);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	do_xx_permutation	
// @@
// @@ INPUT			:
// @@ _source: UI08 *: the source buffer of the permutation.	
// @@
// @@ OUTPUT		:	
// @@
// @@ IO			:	
// @@ _dest: UI08 *: the destination buffer of the permutation.
// @@	
// @@ RETURN VALUE	:	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function perform the bits permutation from the _source
// @@ buffer to the _dest buffer according to the _matrix parameter. 
// @@
// @@ beware: first bit is not zero, but 1, and starts on the left 
// @@ side => bit order : [1 2 3 4 ....]
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::do_xx_permutation (UI08 * _source, UI08 * _dest, int _nb_bit, UI08 * _matrix) {	
	for (int i=1 ; i<=_nb_bit; i++) {
		if (is_set (_source, _matrix[i-1])) {
#if defined (DEBUG_PERMUTATION)			
			cerr << "I am bit " << i << ", my mate is bit " << (int)_matrix[i-1] << " => 1" << endl;
#endif			
			change_bit_value (_dest, i, true);
		}
		else {
#if defined (DEBUG_PERMUTATION)			
			cerr << "I am bit " << i << ", my mate is bit " << (int)_matrix[i-1] << " => 0" << endl;
#endif			
			change_bit_value (_dest, i, false);
		}
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	undo_initial_block_permutation	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to undo the initial block permutation.
// @@ Notice that the result of permutation is in the 'result' member.
// @@
// @@ CONTRACT		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::undo_initial_block_permutation () {
	do_xx_permutation (block, result, BLOCK_SIZE, fp);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	split_block	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to split the internal
// @@ member 'result' into 2 parts composed of 
// @@ 4 bytes each.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::split_block () {
	int i = 0;
	for (i=0 ; i<4 ; i++) {
		Li.atab[i] = result[i];
	}
	for (i=4 ; i<8 ; i++) {
		Ri.atab[i-4] = result[i];
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	merge_block	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to merge the 'Li' and 'Ri'
// @@ internal members into the 'block' member.	
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::merge_block () {
#if defined (__DEBUG_DES__)	
	display_hex_buf ("L16", (UI08 *) &Li.atab[0], 4);
	display_hex_buf ("R16", (UI08 *) &Ri.atab[0], 4);
#endif	
	int i = 0;
	for (i=0 ; i<4 ; i++) {
		block[i] = Li.atab[i];
	}
	for (i=4 ; i<8 ; i++) {
		block[i] = Ri.atab[i-4];
	}
#if defined (__DEBUG_DES__)	
	display_hex_buf ("merge result", (UI08 *) &block[0], 8);	
#endif	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	is_set	
// @@
// @@ INPUT			:
// @@ _cptr: UI08 *: address of the first byte of the buffer whose
// @@   bit is to be changed.	
// @@
// @@ _bit: position of the bit to change (beware: it starts at 1).
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if operation was succesful,
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to set the bit '_bit' from the buffer
// @@ whose address is given by the '_cptr' parameter.
// @@ 
// @@ beware: bit start at 1, and not zero.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool Cdes::is_set (UI08 * _cptr, int _bit) {
	bool bret 	= false;
	_bit 		= _bit - 1;
	int his_byte 	= _bit / 8;
	int his_offset	= _bit % 8;
	UI08 mask	= 0x80 >> his_offset;
	
	if ((_cptr[his_byte] & mask) == mask) {
		bret = true;
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	change_bit_value	
// @@
// @@ INPUT			:
// @@ _cptr: UI08 *: address of the first byte of the buffer whose
// @@   bit is to be changed.	
// @@
// @@ _bit: position of the bit to change (beware: it starts at 1).
// @@
// @@ _set: bool: true if the bit is to be set,
// @@             false otherwise.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to change the bit '_bit' from the buffer
// @@ whose address is given by the '_cptr' parameter according to
// @@ the '_set' parameter : if '_set' is true, then the bit is set,
// @@ otherwise it is reset.
// @@ 
// @@ beware: bit start at 1, and not zero.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::change_bit_value (UI08 * _cptr, int _bit, bool _set) {
	_bit            = _bit - 1;
	int his_byte    = _bit / 8;
	int his_offset  = _bit % 8;
	UI08 mask = 0x80;
	if (_set == true) {
		mask	= mask >> his_offset;
		_cptr[his_byte] = _cptr[his_byte] | mask;
	}
	else { // reset the bit
		mask    = mask >> his_offset;
		mask 	= ~mask; 
		_cptr[his_byte] = _cptr[his_byte] & mask;
	}	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	rotate_left	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	
// @@ _item: SI32&: the data to be rotated
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to perform the rotation of the data
// @@ given by the '_item' parameter toward the left.
// @@
// @@ beware: rotate only 28 bits (bit starting from 1 and from 
// @@         left position)
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::rotate_left (SI32& _item) {
	convert_2_big_endian (_item);
	bool go_out = false;
	if (_item < 0) {
		go_out = true;
	}
	_item = _item << 1;
	if (go_out == true) {
		_item = _item | 0x00000010; // patch bit 28
	}
	convert_2_little_endian (_item);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	permute_choice_1_and_split	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to perform the first 2 steps of the key
// @@ schedule calculation :
// @@ - choose 56 bits out of 64 bits using the 'pc1_left' and 
// @@   'pc1_right' matrix to do the bit choices.
// @@ - assign the result (28 bits for C0, 28 bit for D0) to 
// @@   the 'choice_1_left' and 'choice_1_right' inner members.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::permute_choice_1_and_split () {
	choice_1_left = choice_1_right = 0;
#if defined (__DEBUG_DES__)	
	display_hex_buf ("key", key, 8);
	cerr << "buidling C0" << endl;
#endif	
	do_xx_permutation (key, (UI08 *) &choice_1_left,  CHOICE_1_NB_BIT_HALF, pc1_left);
#if defined (__DEBUG_DES__)	
	cerr << "buidling D0" << endl;
#endif	
	do_xx_permutation (key, (UI08 *) &choice_1_right, CHOICE_1_NB_BIT_HALF, pc1_right);
#if defined (__DEBUG_DES__)	
	display_hex_buf ("C0", (UI08 *) &choice_1_left, 4);
	display_hex_buf ("D0", (UI08 *) &choice_1_right, 4);
#endif	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	aggregate_choice_1	
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to obtain 56 bits using 2 parts composed
// @@ 28 bits. It is used in each stage of the key schedule
// @@ calculation.
// @@ Notice that we obtain 56 bits stored into 8 bytes 
// @@ for convenience.
// @@
// @@ beware: bit start at 1.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::aggregate_choice_1 () {
	memset (appended_choice_1, 0, APPENDED_CHOICE_1_NB_BYTE);
	int i = 0;
	for (i=1 ; i<=CHOICE_1_NB_BIT_HALF ; i++) {
        if (is_set ((UI08 *) &choice_1_left, i)) {
                change_bit_value (appended_choice_1, i, true);
        }
        else {
                change_bit_value (appended_choice_1, i, false);
        }
	}
	for (i=1 ; i<=CHOICE_1_NB_BIT_HALF ; i++) {
		if (is_set ((UI08 *) &choice_1_right, i)) {
			change_bit_value (appended_choice_1, CHOICE_1_NB_BIT_HALF + i, true);
		}
		else {
			change_bit_value (appended_choice_1, CHOICE_1_NB_BIT_HALF + i, false);
		}
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	precalculate_key_schedule	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function performs the D.E.S key schedule calculation
// @@ composed of several steps :
// @@ - using the choice matrixes, choose 2*28 bits among the 64
// @@   bits of the key
// @@ - for each stage :
// @@   - left shift (1 or 2 positions) the 28 bit value
// @@   - generate a 48 bit value using the 2*28 bits resulting 
// @@     from the shift operation.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::precalculate_key_schedule () {
	permute_choice_1_and_split ();
#if defined (__DEBUG_DES__)	       	
char tmp[128];	
#endif
	for (int i=1 ; i<=NB_ROUND ; i++) {
		rotate_left (choice_1_left);
		rotate_left (choice_1_right);
		
		if ((i!=1) && (i!=2) && (i!=9) && (i!=16)) { // all other have must shift twice
			rotate_left (choice_1_left);
			rotate_left (choice_1_right);
		}
#if defined (__DEBUG_DES__)	       	
snprintf (tmp, 128, "C%d", i);
tmp[127] = '\0';
		display_hex_buf (tmp, (UI08 *) &choice_1_left, 4);
snprintf (tmp, 128, "D%d", i);
tmp[127] = '\0';
		display_hex_buf (tmp, (UI08 *) &choice_1_right, 4);
#endif			
		aggregate_choice_1 (); // obtain 56 bits stored into 8 bytes for convenience
		do_xx_permutation (appended_choice_1, kn[i-1], CHOICE_2_NB_BIT, pc2);
#if defined (__DEBUG_DES__)		
snprintf (tmp, 128, "kn[%d]", i);
tmp[127] = '\0';
		display_hex_buf (tmp, kn[i-1], CHOICE_2_NB_BYTE);
#endif		
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	compute_f	
// @@
// @@ INPUT			:
// @@ _data: UI32: The Rn data coming from the right part
// @@        of the nth stage of computation (32 bits).	
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function performs the calculation of f(R,K).
// @@ - R being one of the R0, R1,...Rn and is 32 bits long.
// @@ - K being one of the result of the key schedule calculation. 
// @@ The algorithm is as follows :
// @@ - expand R from 32 to 48 bits using an expansion matrix.
// @@ - xor this expanded R with K.
// @@ - Considering the 48 bits as an input :
// @@   - aggregate 8 blocks of 6 bits.
// @@   - For all of these blocks 
// @@     - let the bit 0 and 5 (first and last) be the row
// @@     - let the bit 1,2,3,4 (center bits) be the column
// @@     - use the row and column along with the S-Box corresponding
// @@       with the current 6 bits block to determine the 4 bit value.
// @@ - peform the last permutation using the 'p32i' matrix.     
// @@ 
// @@ Notice :
// @@ The current step is implicitely stored into 'current' member
// @@ which permits to retrieve K(current)
// @@ result is stored into the 'f_output' member.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::compute_f (UI32 _data) { 
	UI08 expanded[EXPANDED_NB_BYTE];
	UI08 sbox_input[EXPANDED_NB_BYTE];
	// perform  the expansion from 32 to 48 bits of the data
	do_xx_permutation ((UI08 *)&_data, expanded, EXPANDED_NB_BIT, ei);
#if defined (__DEBUG_DES__)
	char tmp[256];
	snprintf (tmp, 256, "%d) E(A)", current);
	tmp[255] = '\0';
	display_hex_buf (tmp, expanded, EXPANDED_NB_BYTE);
#endif	
	int i=0;
	// xor the expanded data with the K(i), i being the current round
	for (i=0 ; i<EXPANDED_NB_BYTE ; i++) {
		sbox_input[i] = expanded[i] ^ kn[current][i];
	}
#if defined (__DEBUG_DES__)
	snprintf (tmp, 256, "%d) E(A) ^ j", current);
	tmp[255] = '\0';
	display_hex_buf (tmp, sbox_input, EXPANDED_NB_BYTE, true);
#endif	

	int his_byte;	
	int his_offset;
	int his_byte_offset;
	int his_box;
	UI08 line		= 0;
	UI08 column		= 0;
	UI08 his_value;
	UI32 sbox_output	= 0;
	UI32 current_output_bit	= 1;	
	bool new_value;
	for (i=0 ; i<48 ; i++) { // for each bit from right to left 
		his_byte   	= i/8;	
		his_offset 	= i % SBOX_INPUT_NB_BIT;
		his_byte_offset	= i % 8;
		new_value  = false;
		if (is_set (&sbox_input[his_byte], his_byte_offset+1) == true) { // starts at bit 1
			new_value = true;
		}
		// remember that the row number is composed of the first and the last bit of current
		// input of the Sbox.
		// and that the column is composed of bits 1,2,3,4.
		// so we collect these 6 bits into a byte for convenience.
		if (his_offset == 0) {  
			change_bit_value (&line, 7, new_value); // bit starts at 1
		}
		else if (his_offset == 1) {
			change_bit_value (&column, 5, new_value);
		}
		else if (his_offset == 2) {
			change_bit_value (&column, 6, new_value);
		}
		else if (his_offset == 3) {
			change_bit_value (&column, 7, new_value);
		}
		else if (his_offset == 4) {
			change_bit_value (&column, 8, new_value);
		}
		else if (his_offset == 5) {
			change_bit_value (&line, 8, new_value);
		}
#if defined (DEBUG_SBOX)
		if (new_value == true) {
			snprintf (tmp, 256, "processing bit %d: his_byte=%d, his_offset=%d, his_byte_offset=%d, value=1", i, his_byte, his_offset, his_byte_offset);
		}
		else {
			snprintf (tmp, 256, "processing bit %d: his_byte=%d, his_offset=%d, his_byte_offset=%d, value=0", i, his_byte, his_offset, his_byte_offset);
		}
		tmp[255] = '\0';
		cerr << tmp << endl;	
#endif

		if ((i!=0) && (his_offset == 5)) { // one line and column retrieved 
			his_box         = (i-1)/SBOX_INPUT_NB_BIT;
			// use the S-BOX matrix 
			his_value  	= si[(his_box * ONE_SBOX_NB_BYTE) + (line * 16) + column]; 
#if defined (DEBUG_SBOX)
			snprintf (tmp, 256, "i=%d => his_box=%d, line=%d, column=%d => value=%d", i, his_box, line, column, his_value);
			tmp[255] = '\0';
			cerr << tmp << endl;	
#endif			
			// reset the line and column for further use
			line = column = 0;
			change_bit_value ((UI08 *)&sbox_output, current_output_bit, is_set(&his_value, 5));
			current_output_bit++;
			change_bit_value ((UI08 *)&sbox_output, current_output_bit, is_set(&his_value, 6));
			current_output_bit++;
			change_bit_value ((UI08 *)&sbox_output, current_output_bit, is_set(&his_value, 7));
			current_output_bit++;
			change_bit_value ((UI08 *)&sbox_output, current_output_bit, is_set(&his_value, 8));
			current_output_bit++;
		}
	}

#if defined (__DEBUG_DES__)
        snprintf (tmp, 256, "%d) C", current);
        tmp[255] = '\0';
        display_hex_buf (tmp, (UI08 *) &sbox_output, 4, true);
#endif	
	// perform the last permutation
	do_xx_permutation ((UI08 *)&sbox_output, (UI08 *)&f_output, F_OUTPUT_NB_BIT, p32i); 
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@	
// @@ FUNCTION		:	do_one_round_crypt	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function performs one round of the D.E.S encipher
// @@ algorithm, one step consisting in :
// @@ - using the value of Rn-1 and Kn (the key schedule), 
// @@   compute f(Rn-1,Kn) using the S-Boxes.
// @@ - xor the value of Ln-1 with f(Rn-1,Kn) and assign the
// @@   result to Rn (except for final stage). 
// @@ - Ln being asigned the value of Rn-1 (except for final stage)
// @@
// @@ NOTICE: 
// @@ - The current round is supposed to be stored into 
// @@   the 'current' member.
// @@ - The Final stage (16th) is different :
// @@   - R16 is asigned the result of xor of f(R15, K16) with L15.
// @@   - L16 is assigned the value of R15.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::do_one_round_crypt () {
	compute_f (Ri.aint);
#if defined (__DEBUG_DES__)
        char tmp[256];
        snprintf (tmp, 256, "L%d", current);
        tmp[255] = '\0';
	display_hex_buf (tmp, (UI08 *)&Li.aint, 4);
	snprintf (tmp, 256, "R%d", current);
	tmp[255] = '\0';
	display_hex_buf (tmp, (UI08 *)&Ri.aint, 4);
#endif

	Rii.aint = Li.aint ^ f_output; // xor Ln with fn
	Lii = Ri;
	// next generation becomes the current generation
	if (current != 15) { 
		Li = Lii;
		Ri = Rii;
	}
	else { // right and left are inverted at the last stage
		Li = Rii;
		Ri = Lii;
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@	
// @@ FUNCTION		:	do_one_round_decrypt	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function performs one round of the D.E.S decipher
// @@ algorithm, one step consisting in :
// @@ - using the value of Rn-1 and Kn (the key schedule), 
// @@   compute f(Rn-1,Kn) using the S-Boxes.
// @@ - xor the value of Ln-1 with f(Rn-1,Kn) and assign the
// @@   result to Rn (except for first stage). 
// @@ - Ln being asigned the value of Rn-1 (except for first stage)
// @@
// @@ NOTICE: 
// @@ - The current round is supposed to be stored into 
// @@   the 'current' member.
// @@
// @@ CONTRACT		:	none	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cdes::do_one_round_decrypt () {
        compute_f (Ri.aint);
#if defined (__DEBUG_DES__)
        char tmp[256];
        snprintf (tmp, 256, "L%d", current);
        tmp[255] = '\0';
        display_hex_buf (tmp, (UI08 *)&Li.aint, 4);
        snprintf (tmp, 256, "R%d", current);
        tmp[255] = '\0';
        display_hex_buf (tmp, (UI08 *)&Ri.aint, 4);
#endif

        Rii.aint = Li.aint ^ f_output;
        Lii = Ri;
        // next generation becomes the current generation
        if (current != 0) {
                Li = Lii;
                Ri = Rii;
        }
        else { // right and left are inverted at the last stage
                Li = Rii;
                Ri = Lii;
        }
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	encrypt	
// @@
// @@ INPUT			:	
// @@ _key: UI08 *: the 64 bit key to be used for the operation.
// @@ 
// @@ _block: UI08 *: the 64 bit block to encipher.
// @@
// @@ _result: UI08 *: the buffer containing the result of the 
// @@    operation.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:
// @@ bool: true if operation was succesful, false otherwise.	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function encipher the 64 bit block given as the 
// @@ '_block' parameter using the 64 bit key given as the
// @@ '_key' parameter, result being stored into the '_result'
// @@ parameter as a 64 bit block.
// @@
// @@ The algorithm consists in :
// @@ - pre-computing the key schedule using the key as an input.
// @@ - perform the initial permutation for the input block.
// @@ - perform 16 rounds of encipherment.
// @@ - undo the initial permutation for the result block.
// @@
// @@ CONTRACT		:	
// @@ - '_key'    parameter must not be a NULL pointer.
// @@ - '_block'  parameter must not be a NULL pointer.	
// @@ - '_result' parameter must not be a NULL pointer.	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool Cdes::encrypt (UI08 * _key, UI08 * _block, UI08 * _result) {
	bool bret = true;
	if ((_key == NULL) || (_block == NULL) || (_result == NULL)) {
		bret = false;
	}
	else {
		key 	= _key;
		block 	= _block;
		result	= _result;	
		precalculate_key_schedule ();
#if defined (__DEBUG_DES__)		
		display_hex_buf ("before", block, 8);
#endif		
		do_initial_block_permutation ();
		split_block ();
		for (current=0 ; current<16 ; current++) {
			do_one_round_crypt ();
		}
		merge_block ();
		undo_initial_block_permutation ();
#if defined (__DEBUG_DES__)		
		display_hex_buf ("after", result, 8);
#endif		
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ FUNCTION		:	decrypt	
// @@
// @@ INPUT			:	
// @@ _key: UI08 *: the 64 bit key to be used for the operation.
// @@ 
// @@ _block: UI08 *: the 64 bit block to decipher.
// @@
// @@ _result: UI08 *: the buffer containing the result of the 
// @@    operation.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:
// @@ bool: true if operation was succesful, false otherwise.	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function decipher the 64 bit block given as the 
// @@ '_block' parameter using the 64 bit key given as the
// @@ '_key' parameter, result being stored into the '_result'
// @@ parameter as a 64 bit block.
// @@
// @@ The algorithm consists in :
// @@ - pre-computing the key schedule using the key as an input.
// @@ - perform the initial permutation for the input block.
// @@ - perform 16 rounds of decipherment.
// @@ - undo the initial permutation for the result block.
// @@
// @@ CONTRACT		:	
// @@ - '_key'    parameter must not be a NULL pointer.
// @@ - '_block'  parameter must not be a NULL pointer.	
// @@ - '_result' parameter must not be a NULL pointer.	
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool Cdes::decrypt (UI08 * _key, UI08 * _block, UI08 * _result) {
	bool bret = true;
	if ((_key == NULL) || (_block == NULL) || (_result == NULL)) {
		bret = false;
	}
	else {
                key     = _key;
                block   = _block;		
                result  = _result;
                precalculate_key_schedule ();
#if defined (__DEBUG_DES__)		
                display_hex_buf ("before", block, 8);
#endif		
		do_initial_block_permutation ();
#if defined (__DEBUG_DES__)		
                display_hex_buf ("after initial permutation", result, 8);
#endif		
		split_block ();
#if defined (__DEBUG_DES__)		
		display_hex_buf ("L16", (UI08 *)&Li.aint, 4);
		display_hex_buf ("R16", (UI08 *)&Ri.aint, 4);
#endif		
		for (current=15 ; current>=0 ; current--) {
			do_one_round_decrypt ();
		}
		merge_block ();
		undo_initial_block_permutation ();
#if defined (__DEBUG_DES__)		
		display_hex_buf ("after", result, 8);
#endif		
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
