// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: des_tool.hpp
// @@ 
// @@ DESCRIPTION:
// @@ implementation of the Data Encryption 
// @@ Standard symetric key algorithm.
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ multiple inclusion problem avoidance
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#ifndef __DES_TOOL_FLAG__
#define __DES_TOOL_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include <cstdio>
#include <cstdlib>
#include <string>
#include <iostream>



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ namespace usage declaration
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
using namespace std;




// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ constants
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define SI08	signed char
#define UI08	unsigned char
#define UI16	unsigned short  
#define UI32	unsigned int
#define SI32	signed int
#define CCPTR	const char *

#define BLOCK_SIZE 					64
#define BLOCK_SIZE_IN_BYTE			8
#define KEY_SIZE_IN_BIT				64
#define KEY_SIZE_IN_BYTE			8
#define CHOICE_1_NB_BYTE			7
#define CHOICE_1_NB_BIT				56
#define CHOICE_2_NB_BIT				48
#define CHOICE_2_NB_BYTE			6
#define CHOICE_1_NB_BIT_HALF		28
#define KEY_SIZE   					8
#define NB_ROUND					16
#define APPENDED_CHOICE_1_NB_BYTE	8
#define EXPANDED_NB_BYTE			6
#define EXPANDED_NB_BIT				48
#define SBOX_INPUT_NB_BIT			6
#define F_OUTPUT_NB_BIT				32
#define ONE_SBOX_NB_BYTE			64			

#if defined (__WIN32_OS__) 
	#define snprintf _snprintf
#endif // #if ! defined (__WIN32_OS__) 


union A32 {
	SI08 atab[4];
      	SI32  aint; 	
};

union A16 {
	UI08 atab[2];
	int ashort;
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cdes	
// @@
// @@ DESCRIPTION	:
// @@ This object permits to cipher/decipher a given 64 bit buffer
// @@ using the "Data Encryption Standard" algorithm.
// @@ 
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
class Cdes
{
private:
	UI08 * 	key;
    UI08 * 	block;
	UI08 * 	result;	
	int    	current;	
	A32		Li;
	A32		Lii;
	A32		Ri;
	A32		Rii;
	int		choice_1_left;
	int		choice_1_right;
	UI32	f_output;
	UI08	appended_choice_1[8]; // only 7 necessary, but 8 is more convenient for computing
	UI08	kn[NB_ROUND][CHOICE_2_NB_BYTE];
	bool 	is_set (UI08 *, int);
	void	change_bit_value (UI08 *, int, bool);	
	void	do_initial_block_permutation ();
	void	undo_initial_block_permutation ();
	void	do_xx_permutation (UI08 *, UI08 *, int, UI08 *);
	void	split_block ();
	void	merge_block ();
	void	do_one_round_decrypt ();
	void	do_one_round_crypt ();
	void	reset ();
	void	compute_f (UI32);
	void	rotate_left (signed int&);
	void	permute_choice_1_and_split ();
	void	aggregate_choice_1 ();
	void	precalculate_key_schedule ();	
	
public:
	Cdes();
	bool	encrypt (UI08 *, UI08 *, UI08 *);		
	bool	decrypt (UI08 *, UI08 *, UI08 *);
	friend void display_hex_buf (string, UI08 *, int, bool);
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ Functions
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void			display_hex_buf (string, UI08 *, int, bool _display_bitfield=false);



#endif // __DES_TOOL_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

