// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: file_mapper.cpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains the object that permits
// @@ to mmap a given file on both win32 or Linux
// @@ OS.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include "../lib/file_mapper.hpp"



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	Cfile_mapper		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ Normal constructor of the Cfile_mapper object.
// @@ it prepares the object to be used.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Cfile_mapper::Cfile_mapper() {
	nb_tick		= 0;
#if defined (__WIN32_OS__) 
	hMapFile	= NULL;
	pBuf		= NULL;
	hFile		= NULL;
#endif // #if defined (__WIN32_OS__) 
#if defined (__LINUX_OS__)
	mmaped_cptr	= NULL;
	mmaped_size	= 0;
#endif // #if defined (__LINUX_OS__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	~Cfile_mapper		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ destructor of the Cfile_mapper object.
// @@ if a file was mmapped, then it is closed
// @@ and all opened descriptors are closed as well.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Cfile_mapper::~Cfile_mapper() {
	close_mmaped_file();
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	close_mmaped_file		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ Destructor of the Cfile_mapper object.
// @@ if a file was mmapped, then it is closed
// @@ and all opened descriptors are closed as well.
// @@
// @@ CONTRACT 		:	none 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cfile_mapper::close_mmaped_file () {
#if defined (__WIN32_OS__) 
		if (pBuf != NULL) {
			UnmapViewOfFile (pBuf);
			pBuf = NULL;
		}

		if ((hMapFile != NULL) && (hMapFile != INVALID_HANDLE_VALUE)) {
			CloseHandle (hMapFile);
			hMapFile = NULL;
		}

		if ((hFile != NULL) && (hFile != INVALID_HANDLE_VALUE)) {
			CloseHandle (hFile);
			hFile = NULL;
		}	
#endif // #if defined (__WIN32_OS__) 

#if defined (__LINUX_OS__)
		if (mmaped_cptr != NULL) {    
			munmap (mmaped_cptr, mmaped_size);
			mmaped_cptr	= NULL;
			mmaped_size	= 0;
		}
#endif // #if defined (__LINUX_OS__)}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	get_last_mmap_error		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	
// @@ _ercode: UI32& : the last error code.
// @@ _ersource: UI32&: the location of the last error (see source code).
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to retrieve the last error related data
// @@ - the error code (often obtained with errno or GetLastError)
// @@ - the error source permitting to know the exact location in
// @@   the source code where the error occured.
// @@
// @@ CONTRACT 		:	none 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cfile_mapper::get_last_mmap_error (UI32& _ercode, UI32& _ersource) {
	_ercode		= error_code;
	_ersource	= error_source;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	get_nb_tick		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ UI32: the number of ticks	that spent before the CreateFile
// @@       function returned, a tick being 250 millisecond.
// @@		This is necessary because sometimes, when a file has
// @@       just been closed and then reopened immediately, the 
// @@       has not finished yet to unmap the previous mapping, and
// @@       thus the mmap fails, the only thing we have to do is 
// @@       retry later, so this function permits to know how many
// @@       ticks it took before the mapping succeeded.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@
// @@ CONTRACT 		:	none 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI32 Cfile_mapper::get_nb_tick () {
	return (nb_tick);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	mmap_file (win32 version)		
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to map to memory
// @@ _open_mode: UI32: can be either 
// @@    - MMAP_READ_ONLY: file will be read only
// @@    - MMAP_RW : file will have read/write privilege.
// @@    - MMAP_RX : file will have read/execution privileges.
// @@
// @@ OUTPUT		:
// @@ _pBuf: UI08 *&: address of the first byte of the file 
// @@                 after it was projected to memory.
// @@ 	
// @@ _file_size: UI32&: size of the file after it was projected 
// @@                    to memory.
// @@
// @@ IO			:	none	
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if success,
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @2 This function permits to map to memory the file whose name
// @@ is given as the '_filename' parameter. 
// @@
// @@ CONTRACT 		: 
// @@ - no file should be currently mapped to memory (or ensure that
// @@   you call 'close_mmaped_file' before. 
// @@ - '_filename' parameter must not be a NULL pointer.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#if defined (__WIN32_OS__) 
	bool Cfile_mapper::mmap_file (CCPTR _filename, UI32 _open_mode, UI08 *& _pBuf, UI32& _file_size, bool _fail_safe) {
		bool bret			= true;
		bool usage_error	= false;
		
		if (_filename == NULL) {
			error_code		= 0;
			error_source	= 99;
			bret			= false;
			usage_error		= true;
		}
		// if a file is already being mmapped, then we consider it as an error.
		else if (hFile != NULL) {
			error_code		= 0;
			error_source	= 100;
			bret			= false;
			usage_error		= true;
		}
		else if (hMapFile != NULL) {
			error_code		= 0;
			error_source	= 101;
			bret			= false;
			usage_error		= true;
		}
		else if (pBuf != NULL) {
			error_code		= 0;
			error_source	= 102;
			bret			= false;
			usage_error		= true;
		}
		else { // file was not already opened
			DWORD access_mode_CreateFile;
			DWORD access_mode_CreateFileMapping;
			DWORD access_mode_MapViewOfFile;
			DWORD create_disposition;

			if (_open_mode == MMAP_READ_ONLY) {
				access_mode_CreateFile			= GENERIC_READ;	
				access_mode_CreateFileMapping	= PAGE_READONLY;
				access_mode_MapViewOfFile		= FILE_MAP_READ;
				create_disposition				= OPEN_EXISTING;
			}
			else if (_open_mode == MMAP_RW) {
				access_mode_CreateFile			= GENERIC_READ | GENERIC_WRITE;
				access_mode_CreateFileMapping	= PAGE_READWRITE;
				access_mode_MapViewOfFile		= FILE_MAP_ALL_ACCESS;
				create_disposition				= OPEN_ALWAYS;
			}
			else if (_open_mode == MMAP_RX) {
				access_mode_CreateFile			= GENERIC_READ | GENERIC_EXECUTE;	
				access_mode_CreateFileMapping	= PAGE_READONLY;
				access_mode_MapViewOfFile		= FILE_MAP_READ;
				create_disposition				= OPEN_ALWAYS;
			}

			if (_fail_safe == false) {
				hFile = CreateFile (_filename, access_mode_CreateFile,0,NULL,create_disposition,FILE_ATTRIBUTE_NORMAL,NULL);
			}
			else {
				nb_tick = 0;
				do {
					hFile = CreateFile (_filename, access_mode_CreateFile,0,NULL,create_disposition,FILE_ATTRIBUTE_NORMAL,NULL);
					if ((hFile != INVALID_HANDLE_VALUE) || (nb_tick > 50)) {
						break;
					}
					else {
						nb_tick++;
						Sleep(250);
					}
				}
				while (hFile == INVALID_HANDLE_VALUE); 
			}
 
			if (hFile == INVALID_HANDLE_VALUE)  {
				hFile			= NULL;
				error_code		= GetLastError ();
				error_source	= 2;
				bret			= false;
			}
			else { // CreateFile was succesfull
				// retrieve the file size
				//if ((_open_mode == MMAP_READ_ONLY) || (_open_mode == MMAP_RX)) {
					_file_size = GetFileSize (hFile, NULL);
				//}
				if (_file_size == 0) { // illegal to mmap a file with a size of zero
					error_code		= GetLastError ();
					error_source	= 74;
					bret = false;
				}
				else if (_file_size < 0) {
					error_code		= GetLastError ();
					error_source	= 6;
					bret			= false;
				}
				else if (_file_size == 0xFFFFFFFF) {
					error_code		= GetLastError ();
					error_source	= 3;
					bret			= false;
				}
				else {
					if ((_open_mode == MMAP_READ_ONLY) || (_open_mode==MMAP_RX)) {
						hMapFile = CreateFileMapping (hFile,NULL,access_mode_CreateFileMapping,0,0,NULL);
					}
					else {
						hMapFile = CreateFileMapping (hFile,NULL,access_mode_CreateFileMapping,0,_file_size,NULL);
					}
					if (hMapFile == NULL/* || hMapFile == INVALID_HANDLE_VALUE*/)  { // failed
						hMapFile		= NULL;
						error_code		= GetLastError ();
						error_source	= 4;
						bret			= false;			
					}
					else { // CreateFileMapping was succesfull
						pBuf = (LPTSTR) MapViewOfFile (hMapFile,	   // handle to map object
											access_mode_MapViewOfFile, // read/write permission
											0,                   
											0,                   
											0/* MMAP_BUF_SIZE */);           

						if (pBuf == NULL) {
							error_code		= GetLastError ();
							error_source	= 5;
							bret			= false;	
						}
						else { // all succeeded, copy obtained pointer to caller's variable
							_pBuf = (UI08 *) pBuf;
						}
					}
				}
			}
		}

		if ((usage_error == false) && (bret == false)) {
			close_mmaped_file ();	
		}
		return (bret);
	}
#endif // #if defined (__WIN32_OS__) 



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cfile_mapper
// @@
// @@ FUNCTION		:	mmap_file (Linux version)		
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to map to memory
// @@ _open_mode: UI32: can be either 
// @@    - MMAP_READ_ONLY: file will be read only
// @@    - MMAP_RW : file will have read/write privilege.
// @@    - MMAP_RX : file will have read/execution privileges.
// @@
// @@ OUTPUT		:
// @@ _pBuf: UI08 *&: address of the first byte of the file 
// @@                 after it was projected to memory.
// @@ 	
// @@ _file_size: UI32&: size of the file after it was projected 
// @@                    to memory.
// @@
// @@ IO			:	none	
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if success,
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @2 This function permits to map to memory the file whose name
// @@ is given as the '_filename' parameter. 
// @@
// @@ CONTRACT 		: 
// @@ - no file should be currently mapped to memory (or ensure that
// @@   you call 'close_mmaped_file' before. 
// @@ - '_filename' parameter must not be a NULL pointer.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#if defined (__LINUX_OS__)
		bool Cfile_mapper::mmap_file (CCPTR _filename, UI32 _open_mode, UI08 *& _pBuf, UI32& _file_size) {
		bool bret = true;

		if (_filename == NULL) {
			error_source	= 1;
			error_code		= 100;
			bret = false;	
		}
		else {
			mapped_filename = _filename;
			int fd = open (_filename, O_RDONLY);
			if (fd < 0) {
				error_code		= errno;
				error_source	= 2;
				bret			= false;
			}
			else { // file opened succesfully
				struct stat sbuf;
				if (stat (_filename, &sbuf) == -1) {
					error_code		= errno;
					error_source	= 3;
					bret			= false;
				}
				else { // file size retrieved succesfully
					_file_size = sbuf.st_size;
					void * ptrtmp = mmap ((caddr_t)0, sbuf.st_size, PROT_READ, MAP_SHARED, fd, 0);
					if (ptrtmp == (caddr_t)(-1)) {
						_pBuf			= NULL;    
						error_code		= errno;
						error_source	= 4;
						bret			= false;
					}
					else {
						mmaped_cptr	= (UI08 *)ptrtmp;
						mmaped_size	= _file_size;    
						_pBuf		= mmaped_cptr;
					}
				}
				
				if (close (fd) == -1) {
					error_code		= errno;
					error_source	= 5;
					bret			= false;
				}
			}
		}
		return (bret);
	}
#endif // #if defined (__LINUX_OS__)



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
