// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: file_mapper.hpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains the object that permits
// @@ to mmap a given file on both win32 or Linux
// @@ OS.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ multiple inclusion problem avoidance
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#ifndef __LIB_FILE_MAPPER_FLAG__
#define __LIB_FILE_MAPPER_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#if defined (__WIN32_OS__) 
	#pragma warning(disable: 4786)
	#pragma warning(disable: 4503)
	#pragma warning(disable: 4291)
#endif // #if defined (__WIN32_OS__) 

	#include <iostream>
	#include <string>
	#include <vector>
	#include <cassert>
	#include <cstdio>
	#include <stack>
	#include <list>
	#include <map>
	#include <exception>
#if defined (__WIN32_OS__) 
	#include <winsock.h>
	#include <Windows.h>
	#include <windows.h>
	#include <winnt.h>
	#include <Commdlg.h>
	#include <commctrl.h>
	#include <Commdlg.h>
	#include <shellapi.h>
	#include <fcntl.h>
	#include <shlobj.h>
	#include <Mmsystem.h>

	#include <io.h>
	#include <aclapi.h>
	#include <lmerr.h>
#endif // #if defined (__WIN32_OS__) 

#if defined (__LINUX_OS__)
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <dirent.h>             // readdir(), etc.
#include <sys/stat.h>           // stat(), etc.
#include <cstring>             // strstr(), etc.
#include <errno.h>
#endif // #if defined (__LINUX_OS__)

#include "../LIB/general.hpp"




// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ namespace usage declaration
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
using namespace std;



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ file mmap possible open mode
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
enum mmap_possible_modes_enum {
	MMAP_READ_ONLY = 0,
	MMAP_RW,
	MMAP_RX,
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS: Cfile_mapper
// @@
// @@ DESCRIPTION:
// @@ This object permits to map a given file to memory
// @@ so that it can be accessed as a buffer, and thus it is 
// @@ faster.
// @@ The object works on both win32 and Linux.
// @@ 
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
class Cfile_mapper
{
	private:
		string	mapped_filename;
		UI32	error_code;
		UI32	error_source;
		UI32	nb_tick;
#if defined (__WIN32_OS__) 
		HANDLE	hFile;
		HANDLE	hMapFile;
		LPCTSTR pBuf;
#endif // #if defined (__WIN32_OS__) 

#if defined (__LINUX_OS__)
		UI08 *  mmaped_cptr;
		UI32    mmaped_size;
#endif // #if defined (__LINUX_OS__)

	public:
		Cfile_mapper();
		~Cfile_mapper();
		bool mmap_file (CCPTR _filename, UI32 _open_mode, UI08 *& _pBuf, UI32& _file_size, bool _fail_safe=false);
		void close_mmaped_file ();
		void get_last_mmap_error (UI32&, UI32&);
		UI32 get_nb_tick ();
};



#endif // __LIB_FILE_MAPPER_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
