// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: general.hpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains all objects, and functions
// @@ that might be used throughout several 
// @@ different projects.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include "../lib/general.hpp"



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	CDuration	
// @@
// @@ FUNCTION		:	CDuration				
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ Normal constructor of the 'CDuration' object.
// @@ It prepares the object before first use by
// @@ calibrating it.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
CDuration::CDuration() {
	LARGE_INTEGER liFrequency;
	QueryPerformanceFrequency(&liFrequency);
	m_llFrequency = liFrequency.QuadPart;

	start(); // Calibration
	stop();
	m_llCorrection = m_liStop.QuadPart-m_liStart.QuadPart;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	CDuration	
// @@
// @@ FUNCTION		:	start				
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to start the precision counter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void CDuration::start() {
	// Ensure we will not be interrupted by any other thread for a while
	// Sleep(0);
	QueryPerformanceCounter(&m_liStart);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	CDuration	
// @@
// @@ FUNCTION		:	stop				
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to stop the precision counter.
// @@ To obtain the elapsed duration, you may use either the
// @@ function 'get_duration' or 'stop_and_format_duration'
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void CDuration::stop() {
	QueryPerformanceCounter(&m_liStop);
}

UI08 CDuration::get_random_UI08 () {
	LARGE_INTEGER m_lirand;
	QueryPerformanceCounter(&m_lirand);
	srand (m_lirand.QuadPart);
	UI08 nb_wait = rand ();
	Sleep (nb_wait);
	return (UI08)(m_lirand.QuadPart);
}

UI32 CDuration::get_random_UI32 () {
	LARGE_INTEGER m_lirand;
	QueryPerformanceCounter(&m_lirand);
	srand (m_lirand.QuadPart);
	UI08 nb_wait = rand ();
	Sleep (nb_wait);
	return (UI32)(m_lirand.QuadPart);
}


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	CDuration	
// @@
// @@ FUNCTION		:	get_duration			
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ double: the elapsed time between the call to the 'start' function
// @@         the unit being the microsecond.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract the time elapsed between the 
// @@ to start and stop. The extracted duration's unit is the 
// @@ microsecond.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
double CDuration::get_duration() const {
	return (double)(m_liStop.QuadPart-m_liStart.QuadPart-m_llCorrection)*1000000.0 / m_llFrequency;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	CDuration	
// @@
// @@ FUNCTION		:	stop_and_format_duration				
// @@
// @@ INPUT			:	
// @@ _msg: CCPTR: the message to use when formatting the elapsed 
// @@       time expressed in microseconds.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the elapsed time between the call to start and
// @@   this function, coming along with the message given by
// @@   the '_msg' parameter.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to do the stop and the get_duration
// @@ in a unique call, the result being a string composed of
// @@ the messages given by the '_msg' parameter, coming along
// @@ with the elapsed time expressed in microseconds.
// @@ NOTICE: if you decide to use this function, you don't need
// @@         to call the 'stop' function, the stop will be done
// @@         by the 'stop_and_format_duration' function itself.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string CDuration::stop_and_format_duration (CCPTR _msg) {
	stop ();
	double elapsed = get_duration ();
	char tmp[MAX_PATH];
	if (elapsed < 1000000.0) {
		snprintf (tmp, 1024, "%s : %lf microseconds", _msg, elapsed);
	}
	else if ((elapsed >= 1000000.0) && (elapsed < 60000000.0)) {
		elapsed = elapsed / 1000000.0;
		snprintf (tmp, 1024, "%s : %lf seconds", _msg, elapsed);
	}
	else {
		elapsed = elapsed / 60000000.0;
		snprintf (tmp, 1024, "%s : %lf minutes", _msg, elapsed);
	}
	tmp[1024 - 1] = '\0';
	return tmp;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	file_is_read_only			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to check.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the file whose name is given by the '_filename'
// @@       parameter is read only, 
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to determine whether the file whose 
// @@ name is given by the '_filename' parameter is read-only or not.
// @@ The result of the test is given as a return value.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool file_is_read_only (CCPTR _filename) {
	bool bret = false;

	DWORD dwAttrs; 
	dwAttrs = GetFileAttributes (_filename); 
	if ((dwAttrs != -1) && (dwAttrs & FILE_ATTRIBUTE_READONLY)) {
		bret = true;
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	remove_read_only_protection			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to check.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the read-only protection for the file whose name
// @@       is given as the '_filename' parameter has been succesfuly
// @@       removed,
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to remove the read-only protection of
// @@ the file whose name is given as the '_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool remove_read_only_protection (CCPTR _filename) {
	bool bret = true;

	DWORD dwAttrs;
	dwAttrs = GetFileAttributes (_filename); 
	if (dwAttrs == -1) {
		bret = false;
	}
	else {
		if (!(dwAttrs & FILE_ATTRIBUTE_READONLY)) {
			bret = false;
		}
		else {
			dwAttrs &= ~FILE_ATTRIBUTE_READONLY;
			if (! SetFileAttributes (_filename, dwAttrs)) {
				int iret = GetLastError();
				bret = false;	
			}
			else {
				bret = true;
			}
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	file_is_hidden			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to check.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the file whose name is given as the '_filename'
// @@       parameter is a hidden file,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to determine whether the file whose
// @@ name is given by the parameter '_filename' is hidden or not.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool file_is_hidden (CCPTR _filename) {
	bool bret = false;

	DWORD dwAttrs; 
	dwAttrs = GetFileAttributes (_filename); 
	if ((dwAttrs != -1) && (dwAttrs & FILE_ATTRIBUTE_HIDDEN)) {
		bret = true;
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	file_is_system			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to check.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the file whose name is given as the '_filename'
// @@       parameter is a system file,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to determine whether the file whose
// @@ name is given by the parameter '_filename' is a 
// @@ system file or not.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool file_is_system (CCPTR _filename) {
	bool bret = false;

	DWORD dwAttrs; 
	dwAttrs = GetFileAttributes (_filename); 
	if ((dwAttrs != -1) && (dwAttrs & FILE_ATTRIBUTE_SYSTEM)) {
		bret = true;
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	remove_hidden_attribute			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to consider.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the hidden attibute was succesfuly removed,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to remove the hidden attribute for
// @@ the file whose name is given as the '_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool remove_hidden_attribute (CCPTR _filename) {
	bool bret = true;

	DWORD dwAttrs;
	dwAttrs = GetFileAttributes (_filename); 
	if (dwAttrs == -1) {
		bret = false;
	}
	else {
		if (!(dwAttrs & FILE_ATTRIBUTE_HIDDEN)) {
			bret = false;
		}
		else {
			dwAttrs &= ~FILE_ATTRIBUTE_HIDDEN;
			if (! SetFileAttributes (_filename, dwAttrs)) {
				int iret = GetLastError();
				bret = false;	
			}
			else {
				bret = true;
			}
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	remove_system_attribute			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to consider.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the system attibute was succesfuly removed,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to remove the system attribute for
// @@ the file whose name is given as the '_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool remove_system_attribute (CCPTR _filename) {
	bool bret = true;

	DWORD dwAttrs;
	dwAttrs = GetFileAttributes (_filename); 
	if (dwAttrs == -1) {
		bret = false;
	}
	else {
		if (!(dwAttrs & FILE_ATTRIBUTE_SYSTEM)) {
			bret = false;
		}
		else {
			dwAttrs &= ~FILE_ATTRIBUTE_SYSTEM;
			if (! SetFileAttributes (_filename, dwAttrs)) {
				int iret = GetLastError();
				bret = false;	
			}
			else {
				bret = true;
			}
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	file_exists			
// @@
// @@ INPUT			:	
// @@ _filename: CCPTR: name of the file to consider.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the file whose name is given as the '_filename'
// @@       parameter exists on the file system,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to check whether the file whose name
// @@ is given as the '_filename' parameter exists on the file
// @@ system or not.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool file_exists (CCPTR _filename) {
	bool bret = false;
	FILE * fp = fopen (_filename, "rb");
	if (fp) {
		bret = true;
		fclose (fp);
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	folder_exists			
// @@
// @@ INPUT			:	
// @@ _folder_name: CCPTR: name of the folder to consider.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the folder whose name is given as the '_folder_name'
// @@       parameter exists on the file system,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to check whether the folder whose name
// @@ is given as the '_folder_name' parameter exists on the file
// @@ system or not.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool folder_exists (CCPTR _folder_name) {
	bool bret = false;
	DWORD attr = GetFileAttributes (_folder_name);
	if (attr != INVALID_FILE_ATTRIBUTES) {
		if (attr & FILE_ATTRIBUTE_DIRECTORY) {
			bret = true;
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	folder_exists			
// @@
// @@ INPUT			:	
// @@ _folder_name: CCPTR: name of the folder to consider.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	
// @@ _size: UI32: extracted size of the file.
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the file size was succesfuly retrieved,
// @@       false otherwise.	
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to retrieve the size of the file whose
// @@ name is given as the '_filename' parameter, the result being
// @@ stored into the result parameter '_size'.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool get_file_size (CCPTR _filename, UI32& _size) {
	bool bret = true;
	_size = 0;

	if (strlen(_filename) > 0) {
		FILE * fp = fopen (_filename, "rb");
		if (fp) {
			fseek (fp, 0, SEEK_END);
			_size = ftell (fp);
			fclose (fp);
		}
		else {
			bret = false;
		}
	}
	else {
		bret = false;
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	move_file			
// @@
// @@ INPUT			:	
// @@ CCPTR: _current_filename: name of the current file.
// @@
// @@ CCPTR: _new_filename: new name of the file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the file '_current_filename' has been 
// @@       succesfully moved to the file '_new_filename'.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to move the file whose name is 
// @@ given as the '_current_filename' parameter, to the
// @@ file whose new name is given as the '_new_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool move_file (CCPTR _current_filename, CCPTR _new_filename) {
	bool bret = true;
	if (file_exists (_new_filename) == true) {
		if (DeleteFile (_new_filename) ==0) {
			bret = false;
		}
	}

	if (bret == true) {
		if (MoveFile (_current_filename, _new_filename) == 0) {
			char tmp[256];
			snprintf (tmp, 256, "MoveFile failed (error: %d)", GetLastError());
			tmp[256 - 1] = '\0';
			MessageBox (NULL, tmp, "", MB_ICONERROR);
			bret = false;
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	copy_mmapped_file			
// @@
// @@ INPUT			:	
// @@ _buf: UI08 *: address of the first byte of the mmapped file.
// @@
// @@ _size: UI32: size of the mmapped file.
// @@
// @@ _dest_filename: CCPTR: name of the new file that must has
// @@   the same content as the mmapped file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the copy was succesful,
// @@       false otherwise.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to create a copy of the file that was
// @@ previously mmapped, the content of the file to copy can be 
// @@ found from the '_buf' parameter, the file size being given
// @@ by the '_size' parameter, the name of the copied file is
// @@ given by the '_dest_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool copy_mmapped_file (UI08 * _buf, UI32 _size, CCPTR _dest_filename) {
	bool bret = true;
	FILE * fp_dest = fopen (_dest_filename, "wb");
	if (! fp_dest) {
		bret = false;
	}
	else {
		if (fwrite (_buf, 1, _size, fp_dest) != _size) {
			bret = false;	
		}

		if (fclose (fp_dest) != 0) {
			bret = false;
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	convert_2_windowed_name			
// @@
// @@ INPUT			:
// @@ _name: CCPTR: the original file/folder name
// @@ 
// @@ _max_size: UI32: maximum size of the file/folder name	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the file/folder name reduced to fit into the 	
// @@	      window whose size is given as the '_max_size' parameter.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to generate a filename whose length
// @@ is reduced to fit into the '_max_size' window.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string convert_2_windowed_name (CCPTR _name, UI32 _max_size) {
	string modified_name;
	UI32 size = strlen (_name);
	if (size < _max_size) {
		modified_name = _name;	
	}
	else {
		UI32 block_size = (_max_size - 2) / 2;
		UI32 start;
		for (start=0 ; start < block_size ; start++) {
			modified_name += _name[start];	
		}
		modified_name += "..";
		for (start=size-block_size ; start <size ; start++) {
			modified_name += _name[start];
		}
	}
	return (modified_name);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	extract_extensionless_filename			
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to work on.
// @@ 
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the filename without it's extension.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to generate the filename without
// @@ it's extension, starting from the original filename
// @@ given by the '_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string extract_extensionless_filename (CCPTR _filename) {
	string result;
	UI32 size = strlen (_filename);
	UI32 stop;
	for (stop=0 ; stop < size ; stop++) {
		if (_filename[stop] == '.') {
			break;
		}
		else {
			result += _filename[stop];
		}
	}
	return (result);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	retrieve_file_xxx_time			
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to work on.
// @@ 
// @@ _type: UI32: determine what to retrieve out of :
// @@ - CREATE_ACCESS	: retrieve creation time and date
// @@ - WRITE_ACCESS	: retrieve last write time and date
// @@ - READ_ACCESS		: retrieve last read time and date
// @@
// @@ OUTPUT		:	
// @@ _year: UI32: the year related to the file.
// @@ 
// @@ _month: UI32: the month related to the file.
// @@
// @@ _day: UI32: the day related to the file
// @@
// @@ _hour: UI32: the hour related to the file
// @@
// @@ _minute: UI32: the minute related to the file
// @@
// @@ _second: UI32: the second related to the file
// @@
// @@ _ercode: UI32: the error code if any error occured
// @@
// @@ _ersource: UI32: the error source if any error occured.
// @@ 
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if operation was succesful,
// @@       false otherwise.
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract either the creation date and 
// @@ time, or the last read or write date and time, depending on the
// @@ '_type' parameter, of the file whose name is given by the 
// @@ '_filename' parameter. If any error occurs, the error reason
// @@ and location is returned via the arguments '_ercode' and
// @@ '_ersource'.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool retrieve_file_xxx_time (const char * _filename, UI32& _year, UI32& _month, UI32& _day, UI32& _hour, UI32& _minute, UI32& _second, UI32& _millisecond, UI32 _type, UI32& _ercode, UI32& _ersource) {
	bool bret = true;
    FILETIME ftCreate, ftAccess, ftWrite;
    SYSTEMTIME stUTC, stLocal;
	
	HANDLE hFile = CreateFile(_filename,      // file to open
					   GENERIC_READ,          // open for reading
					   FILE_SHARE_READ,       // share for reading
					   NULL,                  // default security
					   OPEN_EXISTING,         // existing file only
					   FILE_ATTRIBUTE_NORMAL, // normal file
					   NULL);                 // no attr. template

	if (hFile == INVALID_HANDLE_VALUE) {
		_ersource = 1;
		_ercode = GetLastError ();
		bret = false;
	}
	else { // file opened succesfully
		// Retrieve the file times for the file.
		if (!GetFileTime(hFile, &ftCreate, &ftAccess, &ftWrite)) {
			_ercode = GetLastError ();
			_ersource = 2;
			bret = false;
		}
		else { // retrieval succesfull, let's retrieve data depending on the access mode 
			switch (_type) {
				case CREATE_ACCESS:
					FileTimeToSystemTime(&ftCreate, &stUTC);
					SystemTimeToTzSpecificLocalTime(NULL, &stUTC, &stLocal);
					break;

				case WRITE_ACCESS:
					FileTimeToSystemTime(&ftWrite, &stUTC);
					SystemTimeToTzSpecificLocalTime(NULL, &stUTC, &stLocal);
					break;

				case READ_ACCESS:
					FileTimeToSystemTime(&ftAccess, &stUTC);
					SystemTimeToTzSpecificLocalTime(NULL, &stUTC, &stLocal);
					break;

				default: // unknown file access type
					bret = false;
			}

			if (bret == true) {
				_year			= stLocal.wYear;
				_month			= stLocal.wMonth;
				_day			= stLocal.wDay;
				_hour			= stLocal.wHour;
				_minute			= stLocal.wMinute;
				_second			= stLocal.wSecond;
				_millisecond	= stLocal.wMilliseconds;
			}
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	retrieve_file_creation_time			
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to work on.
// @@ 
// @@ OUTPUT		:	
// @@ _year: UI32: the year related to the file.
// @@ 
// @@ _month: UI32: the month related to the file.
// @@
// @@ _day: UI32: the day related to the file
// @@
// @@ _hour: UI32: the hour related to the file
// @@
// @@ _minute: UI32: the minute related to the file
// @@
// @@ _second: UI32: the second related to the file
// @@
// @@ _ercode: UI32: the error code if any error occured
// @@
// @@ _ersource: UI32: the error source if any error occured.
// @@ 
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if operation was succesful,
// @@       false otherwise.
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract the creation date and 
// @@ time, of the file whose name is given by the 
// @@ '_filename' parameter. If any error occurs, the error reason
// @@ and location is returned via the arguments '_ercode' and
// @@ '_ersource'.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool retrieve_file_creation_time (const char * _filename, UI32& _year, UI32& _month, UI32& _day, UI32& _hour, UI32& _minute, UI32& _second, UI32& _millisecond, UI32& _retcode, UI32& _ersource) {
	return (retrieve_file_xxx_time (_filename, _year, _month, _day, _hour, _minute, _second, _millisecond, CREATE_ACCESS, _retcode, _ersource));
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	retrieve_file_last_read_time			
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to work on.
// @@ 
// @@ OUTPUT		:	
// @@ _year: UI32: the year related to the file.
// @@ 
// @@ _month: UI32: the month related to the file.
// @@
// @@ _day: UI32: the day related to the file
// @@
// @@ _hour: UI32: the hour related to the file
// @@
// @@ _minute: UI32: the minute related to the file
// @@
// @@ _second: UI32: the second related to the file
// @@
// @@ _ercode: UI32: the error code if any error occured
// @@
// @@ _ersource: UI32: the error source if any error occured.
// @@ 
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if operation was succesful,
// @@       false otherwise.
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract the last read date and 
// @@ time, of the file whose name is given by the 
// @@ '_filename' parameter. If any error occurs, the error reason
// @@ and location is returned via the arguments '_ercode' and
// @@ '_ersource'.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool retrieve_file_last_read_time (const char * _filename, UI32& _year, UI32& _month, UI32& _day, UI32& _hour, UI32& _minute, UI32& _second, UI32& _millisecond, UI32& _retcode, UI32& _ersource) {
	return (retrieve_file_xxx_time (_filename, _year, _month, _day, _hour, _minute, _second, _millisecond, READ_ACCESS, _retcode, _ersource));
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	retrieve_file_last_write_time			
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to work on.
// @@ 
// @@ OUTPUT		:	
// @@ _year: UI32: the year related to the file.
// @@ 
// @@ _month: UI32: the month related to the file.
// @@
// @@ _day: UI32: the day related to the file
// @@
// @@ _hour: UI32: the hour related to the file
// @@
// @@ _minute: UI32: the minute related to the file
// @@
// @@ _second: UI32: the second related to the file
// @@
// @@ _ercode: UI32: the error code if any error occured
// @@
// @@ _ersource: UI32: the error source if any error occured.
// @@ 
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if operation was succesful,
// @@       false otherwise.
// @@	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract the last write date and 
// @@ time, of the file whose name is given by the 
// @@ '_filename' parameter. If any error occurs, the error reason
// @@ and location is returned via the arguments '_ercode' and
// @@ '_ersource'.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool retrieve_file_last_write_time (const char * _filename, UI32& _year, UI32& _month, UI32& _day, UI32& _hour, UI32& _minute, UI32& _second, UI32& _millisecond, UI32& _retcode, UI32& _ersource) {
	return (retrieve_file_xxx_time (_filename, _year, _month, _day, _hour, _minute, _second, _millisecond, WRITE_ACCESS, _retcode, _ersource));
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	enable_control			
// @@
// @@ INPUT			:	
// @@ _hwnd: HWND: the handler of the window of the control.
// @@
// @@ _idc: long: the unique identifier of the control in it's window. 
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to enable the control whose unique
// @@ identifier is given by the '_idc' parameter, and which
// @@ pertains to the window whose handle is given by the '_hwnd'
// @@ parameter. The control is then enabled, and thus the user 
// @@ can interact with it.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void enable_control (HWND _hwnd, long _idc) {
	HWND button_hwnd = GetDlgItem (_hwnd, _idc); 
	EnableWindow (button_hwnd, TRUE);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	disable_control			
// @@
// @@ INPUT			:	
// @@ _hwnd: HWND: the handler of the window of the control.
// @@
// @@ _idc: long: the unique identifier of the control in it's window. 
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to disable the control whose unique
// @@ identifier is given by the '_idc' parameter, and which
// @@ pertains to the window whose handle is given by the '_hwnd'
// @@ parameter. The control is then disabled, and thus the user 
// @@ can not interact with it, even though it is visible.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void disable_control (HWND _hwnd, long _idc) {
	HWND button_hwnd = GetDlgItem (_hwnd, _idc); 
	EnableWindow (button_hwnd, FALSE);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	dump_mmapped_file			
// @@
// @@ INPUT			:	
// @@ _dump_filename: CCPTR: name of the dump file to create.
// @@
// @@ _ptr: UI08 *: address of the first byte of the mmapped file.
// @@ 
// @@ _size: UI32: size of the mmaped file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the dump was succesful, false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to dump the content of a mmapped file
// @@ to a file on the file system.
// @@ The address of the first byte of the mmapped file is given
// @@ by the '_ptr' parameter, the size of the mmaped file is given
// @@ by the '_size' parameter, and the dump must be saved into the 
// @@ file whose name is given by the '_dump_filename' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool dump_mmapped_file (CCPTR _dump_filename, UI08 * _ptr, UI32 _size) {
	bool bret = true;
	FILE * fp = fopen (_dump_filename, "wb");
	if (! fp) {
		bret = false;
	}
	else {
		if (fwrite (_ptr, 1, _size, fp) != _size) {
			bret = false;
		}
		fclose (fp);
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	extract_folder_from_filename			
// @@
// @@ INPUT			:
// @@ _filename: CCPTR: name of the file to use for extraction.	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the result of the extraction.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract the folder of a given filename.
// @@ For example : "C:\\abc\\cde\\toto.txt" -> "C:\\abc\\cde\\"
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string extract_folder_from_filename (CCPTR _filename) {
	string result;
	UI32 size = strlen (_filename);
	if (size > 0) {
		UI32 start;
		for (start=size-1 ; start>=0 ; start--) {
			if (_filename[start] == '\\') {
				break;		
			}
		}

		for (UI32 i=0 ; i<=start ; i++) {
			result += _filename[i];	
		}
	}
	return (result);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	md5_create_folder			
// @@
// @@ INPUT			:
// @@ _root: CCPTR: folder from where to create the md5 folder.
// @@
// @@ _md5: md5 sum of the file whose folder is to be created.
// @@
// @@ OUTPUT		:	
// @@ _abs_folder_name: string: the name of the absolute folder created.
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the MD5 folder creation was succesful,	
// @@	    false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to create the MD5 folder of a file whose 
// @@ md5 sum is given by the '_md5' parameter, the md5 folder should
// @@ be created from the '_root' folder, and the absolute name of
// @@ the folder created is saved into the '_abs_folder_name' 
// @@ parameter.
// @@ The MD5 folder is the ventilated folder as follows 
// @@ (for the example we use a shorter MD5 sum) :
// @@ "F46A6" -> F/4/6/A/6
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool md5_create_folder (CCPTR _root, CCPTR _md5, string& _abs_folder_name) {
	bool bret = true;
	if (strlen (_md5) != 32) {
		bret = false;
	}
	else {
		if (folder_exists (_root) == false) {
			if (create_folder(_root) == false) {
				bret = false;
			}
		}

		if (bret == true) {
			UI32 i=0;
			_abs_folder_name = _root;
			for (i=0 ; i<32 ; i=i+1) {
				_abs_folder_name = _abs_folder_name + "\\" + _md5[i];
				if (folder_exists (_abs_folder_name.c_str()) == false) {
					if (create_folder (_abs_folder_name.c_str()) == false) {
						bret = false;
						break;
					}
				}
			}
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	create_all_folders_of_path			
// @@
// @@ INPUT			:
// @@ _path: CCPTR: name of the path to create.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the folders were succesfuly created, 
// @@       false otherwise.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to create all the folders along
// @@ the path that is given as the '_path' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool create_all_folders_of_path (CCPTR _path) {
	bool bret = true;
	UI32 size = strlen (_path);
	string current;
	UI32 i;
	UI32 last_size = 0;
	UI32 current_size;
	if ((size < 3) || (_path[1] != ':') || (_path[2] != '\\')) {
		bret = false;
	}
	else {	
		for (i=0 ; i<size ; i++) {
			if (i < 3) {
				current += _path[i];
			}
			else {
				if ((_path[i] == '\\') || (_path[i] == '/')) {
					current_size = current.size();
					if ((current_size > 0) && (current_size > last_size)) {
						bret = create_folder (current.c_str());
						if (bret == false) {
							break;
						}
						else {
							last_size = current_size;
						}
					}
				}
				current += _path[i];
			}
		}
	}

	if ((bret == true) && (current.size() > last_size)) {
		bret = create_folder (current.c_str());	
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	create_folder			
// @@
// @@ INPUT			:
// @@ _folder_name: CCPTR: name of the folder to create.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the folder was succesfuly created, 
// @@       false otherwise.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to create the folder whose name
// @@ is given as the '_folder_name' parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool create_folder (CCPTR _folder_name) {
	bool bret = true;
	if (folder_exists (_folder_name) == false) {
		if (CreateDirectory (_folder_name, NULL) == 0) {
			bret = false;
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	develop_md5_folder_name			
// @@
// @@ INPUT			:
// @@ _abs_root: CCPTR: abolute folder name from where to start
// @@
// @@ _rel_root: CCPTR: relative folder name from where to start
// @@
// @@ _md5: CCPTR: md5 sum of the file whose folder is to be developped.
// @@
// @@ OUTPUT		:	
// @@ _rel_folder_name: string: developped relative folder name.	
// @@
// @@ _abs_folder_name: string: developped absolute folder name.	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to generate the developped md5 folder
// @@ name from the root folder whose absolute name is given by
// @@ the '_abs_root' parameter, and whose relative name is given
// @@ by the '_rel_root' parameter, the md5 sum being given by the
// @@ '_md5' parameter. 
// @@ The developped folder name is saved into the '_rel_folder_name'
// @@ for the relative version, and into the '_abs_folder_name' 
// @@ parameter for the absolute version. 
// @@ The MD5 folder is the ventilated folder as follows 
// @@ (for the example we use a shorter MD5 sum) :
// @@ "F46A6" -> F/4/6/A/6
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void develop_md5_folder_name (CCPTR _abs_root, CCPTR _rel_root, CCPTR _md5, string& _rel_folder_name, string& _abs_folder_name) {
	_abs_folder_name = _abs_root;
	_rel_folder_name = _rel_root;
	for (UI32 i=0 ; i<32 ; i=i+1) {
		_abs_folder_name = _abs_folder_name + "\\" + _md5[i];
		_rel_folder_name = _rel_folder_name + "/" + _md5[i];
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	md5_folder_exists			
// @@
// @@ INPUT			:
// @@ _abs_root: CCPTR: abolute folder name from where to start
// @@
// @@ _rel_root: CCPTR: relative folder name from where to start
// @@
// @@ _md5: CCPTR: md5 sum of the file whose folder is to be developped.
// @@
// @@ OUTPUT		:	
// @@ _rel_folder_name: string: developped relative folder name.	
// @@
// @@ _abs_folder_name: string: developped absolute folder name.	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if the developped MD5 folder exists,
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to generate the developped md5 folder
// @@ name from the root folder whose absolute name is given by
// @@ the '_abs_root' parameter, and whose relative name is given
// @@ by the '_rel_root' parameter, the md5 sum being given by the
// @@ '_md5' parameter. 
// @@ The developped folder name is saved into the '_rel_folder_name'
// @@ for the relative version, and into the '_abs_folder_name' 
// @@ parameter for the absolute version. 
// @@ The MD5 folder is the ventilated folder as follows 
// @@ (for the example we use a shorter MD5 sum) :
// @@ "F46A6" -> F/4/6/A/6
// @@ Of course the function also permits to determine whether this
// @@ md5 folder exists or not.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool md5_folder_exists (CCPTR _abs_root, CCPTR _rel_root, CCPTR _md5, string& _rel_folder_name, string& _abs_folder_name) {
	bool bret = false;
	develop_md5_folder_name (_abs_root, _rel_root, _md5, _rel_folder_name, _abs_folder_name);
	bret = folder_exists (_abs_folder_name.c_str());
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cbar	
// @@
// @@ FUNCTION		:	Cbar			
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ Normal constructor of the 'Cbar' object.
// @@ it resets necessary fields to their initial value.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Cbar::Cbar() {
	hwnd	= 0;
	max		= 0;
	idc		= 0;	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cbar	
// @@
// @@ FUNCTION		:	init			
// @@
// @@ INPUT			:	
// @@ _hwnd: HWND: handle of the window where the bar is located.
// @@
// @@ _idc: int: unique identifier of the progress bar.
// @@ 
// @@ _max: int: maximum position of the progress bar.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to prepare the progress bar
// @@ before it's first use.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cbar::init (HWND _hwnd, int _idc, int _max) {
	hwnd	= _hwnd;
	max		= _max;
	idc		= _idc;
	current	= 0;
	reset();
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cbar	
// @@
// @@ FUNCTION		:	incr			
// @@
// @@ INPUT			:	
// @@ _nb_step: SI32: the number of steps to use to increment
// @@                 the position of the progress bar.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to move the progress bar of 
// @@ '_nb_step' positions toward the right.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cbar::incr (SI32 _nb_step) {
	current += _nb_step;
	SendDlgItemMessage (hwnd, idc, PBM_SETPOS, current, 0);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cbar	
// @@
// @@ FUNCTION		:	reset			
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to bring the progress bar back
// @@ to it's initial position on the left.
// @@ beware though that the bar is still useable even after
// @@ a reset (idc and hwnd members are not reseted)
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cbar::reset() {
	SendDlgItemMessage (hwnd, idc, PBM_SETRANGE32, 0, max); 
	current = 0;
	SendDlgItemMessage (hwnd, idc, PBM_SETPOS, current, 0);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	opa_result_2_string			
// @@
// @@ INPUT			:
// @@ _opa_result: UI08: the result of the last operation.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the last operation as a human-readable string.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function converts the '_opa_result' parameter into
// @@ a human-readable string.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string opa_result_2_string (UI08 _opa_result) {
	string s;
	switch (_opa_result) {
		case OPA_NOT_DONE:		s = "Not Done";		break;
		case OPA_DONE:			s = "Done";			break;
		case OPA_FAILED:		s = "Failed";		break;
		case OPA_NOT_FOUND:		s = "Not Found";	break;
		case OPA_USED:			s = "Used";			break;
		case OPA_NOT_USED:		s = "Not Used";		break;
		case OPA_NONE:			s = "None";			break;
		case OPA_YES:			s = "Yes";			break;
		case OPA_NO:			s = "No";			break;

		default:				s = "<unknown case>";
	}
	return (s);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	file_type_2_string			
// @@
// @@ INPUT			:
// @@ _type: UI32: the type of the file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the type of the file converted into a 
// @@         human-readable string.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function converts the '_type' parameter into
// @@ a human-readable string. This parameter being the
// @@ numeric value that uniquely identifies a given file type.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string file_type_2_string (UI32 _type) {
	string s;
	switch (_type) {
		case FILE_TYPE_NOT_KNOWN:				s = "NOT_KNOWN";			break;
		case FILE_TYPE_JPG:						s = "JPG";					break;
		case FILE_TYPE_GIF:						s = "GIF";					break;	
		case FILE_TYPE_BMP:						s = "BMP";					break;
		case FILE_TYPE_MPEG:					s = "MPEG";					break;
		case FILE_TYPE_MP3:						s = "MP3";					break;
		case FILE_TYPE_WMV:						s = "WMV";					break;
		case FILE_TYPE_TXT:						s = "TXT";					break;
		case FILE_TYPE_TXT_UTF8:				s = "TXT_UTF8";				break;
		case FILE_TYPE_TXT_UNICODE:				s = "UNICODE";				break;
		case FILE_TYPE_TXT_UNICODE_BIG_ENDIAN:	s = "UNICODE_BIG_ENDIAN";	break;
		case FILE_TYPE_EXE:						s = "EXE";					break;			
		case FILE_TYPE_ZIP:						s = "ZIP";					break;	
		case FILE_TYPE_AVI:						s = "AVI";					break;
		case FILE_TYPE_ASF:						s = "ASF";					break;
		case FILE_TYPE_KROSOFT:					s = "MICROSOFT";			break;

		default: s = "<undefined>";
	}
	return (s);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
