// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: general.hpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains all objects, and functions
// @@ that might be used throughout several 
// @@ different projects.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ multiple inclusion problem avoidance
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#ifndef __EXOSTEG_GENERAL_FLAG__
#define __EXOSTEG_GENERAL_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include <cstdio>
#include <cstdlib>
#if defined (__WIN32_OS__)
	#pragma warning(disable: 4786)
	#pragma warning(disable: 4503)
	#pragma warning(disable: 4291)
#endif // #if defined (__WIN32_OS__) 
#if defined (__WIN32_OS__) 
	#include <windows.h>
#endif // #if defined (__WIN32_OS__) 
#include <string>
#include <vector>
#include <commctrl.h>
#include <Commdlg.h>
#include "../lib_plugin/data_types.hpp"
#include "../lib_plugin/general_plugin.hpp"



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ namespace usage declaration
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
using namespace std;


#define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
#define snprintf _snprintf



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ constants related to file mapping 
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
enum all_possible_access_modes_enum {
	CREATE_ACCESS=54,
	WRITE_ACCESS,
	READ_ACCESS,
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS: CDuration
// @@
// @@ DESCRIPTION:
// @@ This object permits to perform high performance time
// @@ measurements under win32 platform.
// @@ The accuracy is the microseconds.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
class CDuration
{
protected:
	LARGE_INTEGER m_liStart;
	LARGE_INTEGER m_liStop;

	LONGLONG m_llFrequency;
	LONGLONG m_llCorrection;

public:
	CDuration();

	void	start();
	void	stop();
	double	get_duration() const;
	string	stop_and_format_duration (CCPTR);
	UI08	get_random_UI08 ();
	UI32	get_random_UI32 ();
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS: Cbar
// @@
// @@ DESCRIPTION:
// @@ This object permits to animate a progress bar
// @@ without knowing how it is actually done to move it.
// @@ Thus it is an abstraction layer.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
class Cbar
{
private:
	int current;
	int max;
	HWND hwnd;
	int	idc;

public:
	Cbar();
	void init (HWND, int, int);
	void incr (SI32 _nb_step=1);
	void reset();
	int get_current () {
		return (current);
	}
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ general usage functions
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool	file_exists						(CCPTR);
bool	folder_exists					(CCPTR);
void	display_result					(CCPTR, UI32, UI32);
string	err_2_string					(SI32);
string	opa_result_2_string				(UI08);
bool	create_folder					(CCPTR);
bool	create_all_folders_of_path		(CCPTR);
bool	md5_folder_exists				(CCPTR, CCPTR, CCPTR, string&, string&);
bool	md5_create_folder				(CCPTR, CCPTR, string&);
void	develop_md5_folder_name			(CCPTR, CCPTR, CCPTR, string&, string&);
string	extract_folder_from_filename	(CCPTR);
string	file_type_2_string				(UI32);
bool	copy_mmapped_file				(UI08 *, UI32, CCPTR);
bool	dump_mmapped_file				(CCPTR, UI08 *, UI32);
string	convert_2_windowed_name			(CCPTR, UI32);
void	enable_control					(HWND, long);
void	disable_control					(HWND, long);
bool	retrieve_file_last_write_time	(CCPTR, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&);
bool	retrieve_file_creation_time		(CCPTR, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&);
bool	retrieve_file_last_read_time	(CCPTR, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&, UI32&);
string	extract_extensionless_filename	(CCPTR);
// UI32	compute_file_size				(CCPTR); // DANGEROUS !
bool	move_file						(CCPTR, CCPTR);
bool	file_is_read_only				(CCPTR);
bool	remove_read_only_protection		(CCPTR);
bool	file_is_hidden					(CCPTR);
bool	remove_hidden_attribute			(CCPTR);
bool	file_is_system					(CCPTR);
bool	remove_system_attribute			(CCPTR);
bool	get_file_size					(CCPTR, UI32&);



#endif // __EXOSTEG_GENERAL_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
