// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: log.cpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains the definition of the 
// @@ log facility that permits to log events and
// @@ to associate a time and date, coming along
// @@ with a severity level for each event.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include "../LIB/log.hpp"



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cone_line
// @@
// @@ FUNCTION		:	insert_date
// @@
// @@ INPUT			:	
// @@ - _fp: FILE * : pointer on an opened valid file.
// @@ - _date: CCPTR: pointer on a string containing the current date
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to insert the date in a fixed
// @@ format into the current line.
// @@
// @@ CONTRACT		: 
// @@ - '_fp' must be a valid opened file's pointer
// @@ - '_date' must not be a NULL pointer
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cone_line::insert_date (FILE * _fp, CCPTR _date) {
	int nb_2_do = strlen (_date);
	if (nb_2_do > DATE_LENGTH) {
		nb_2_do = DATE_LENGTH;	
	}
	fwrite (_date, 1, nb_2_do, _fp);
	fflush (_fp);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cone_line
// @@
// @@ FUNCTION		:	insert_level
// @@
// @@ INPUT			:	
// @@ - _fp: FILE * : pointer on an opened valid file.
// @@ - _level: CCPTR: pointer on a string containing the current date
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to insert the severity level in a fixed
// @@ format into the current line.
// @@ Notice that the severity level may be chosen out of :
// @@ - NORMAL_LEVEL
// @@ - WARNING_LEVEL
// @@ - ERROR_LEVEL
// @@ - ADMIN_LEVEL
// @@ - DEBUG_LEVEL
// @@ - NONE_LEVEL
// @@
// @@ CONTRACT		:
// @@ - '_fp' must be a valid opened file's pointer
// @@ - '_level' must not be a NULL pointer
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cone_line::insert_level (FILE * _fp, CCPTR _level) {
	int nb_2_do = strlen (_level);
	if (nb_2_do > LEVEL_LENGTH) {
		nb_2_do = LEVEL_LENGTH;	
	}
	fwrite (_level, 1, nb_2_do, _fp);
	fflush (_fp);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cone_line
// @@
// @@ FUNCTION		:	insert_body
// @@
// @@ INPUT			:	
// @@ - _fp: FILE * : pointer on an opened valid file.
// @@ - _level: CCPTR: pointer on a string containing the current date
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public	
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to insert the severity level in a fixed
// @@ format into the current line.
// @@
// @@ CONTRACT		:
// @@ - '_fp' must be a valid opened file's pointer
// @@ - '_level' must not be a NULL pointer
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cone_line::insert_body (FILE * _fp, CCPTR _body) {
	int nb_2_do = strlen (_body);
	if (nb_2_do > LINE_LENGTH) {
		nb_2_do = LINE_LENGTH;
	}
	fwrite (_body, 1, nb_2_do, _fp);
	fflush (_fp);
	int remaining = LINE_LENGTH - nb_2_do;
	if (remaining > 0) {
		char vide = ' ';
		for (int i=0 ; i<remaining ; i++) {
			fwrite (&vide, 1, 1, _fp);
		}
		fflush (_fp);
	}
	insert_terminator (_fp);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Cone_line
// @@
// @@ FUNCTION		:	insert_terminator
// @@
// @@ INPUT			:	
// @@ - _fp: FILE * : pointer on an opened valid file.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none	
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	private
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to end the current line by inserting
// @@ an end of line character sequence, that is OS dependant.
// @@
// @@ CONTRACT		: 
// @@ - '_fp' must be a valid opened file's pointer
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Cone_line::insert_terminator (FILE * _fp) {
	fwrite (terminator.c_str(), 1, terminator.size (), _fp);
	fflush (_fp);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-
// @@
// @@ FUNCTION		:	display_file_2_log	
// @@
// @@ INPUT			:
// @@ - _filename: CCPTR: name of the destination file.
// @@
// @@ OUTPUT		:	
// @@
// @@ IO			:	
// @@ - _logger: Clog : instance of the log facility to use.
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	-
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to dump the content of a given file
// @@ into the current log file using the log facility given as
// @@ a parameter.
// @@
// @@ CONTRACT 		: 
// @@ - '_filename' must not be a NULL pointer
// @@ - '_filename' must be the name of a valid and existing file.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void display_file_2_log (CCPTR _filename, Clog& _logger) {
	int ic;
	FILE * fp = fopen (_filename, "rb");
	if (! fp) {
		_logger.log_that (ERROR_LEVEL, "+++ failed to visualize file (fopen failed) : ", _filename);	
	}
	else {
		string total = "";
		ic = fgetc (fp);
		while (ic != EOF) {
			total += ic;
			ic = fgetc (fp);
		}
		fclose (fp);
		_logger.log_that (DEBUG_LEVEL, "file content : ", total.c_str());
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	Clog		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ Normal constructor of the Clog object.
// @@
// @@ CONTRACT 		:	none	 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Clog::Clog() {
	reset ();
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	~Clog		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ destructor of the Clog object.
// @@
// @@ CONTRACT 		:	none	 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Clog::~Clog() {
	close_log ();
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	check_termination	
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	private
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to determine whether the current line
// @@ is the last line of the rolling log file. If it is the case
// @@ then the file is rewinded so that the next line logged will
// @@ be located at the top of the log file.
// @@
// @@ CONTRACT 		: none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::check_termination () {
	current_line++;
	if (current_line >= LOG_MAX_NB_LINE) {
		rewind (fp);
		current_line = 1;
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	open_log	
// @@
// @@ INPUT			:
// @@ _filename: string : name of the log file to open.
// @@ _error_log_ptr: address of the error log, can be NULL if
// @@    error log is not used.
// @@ - _credential: string: symbolic name of the log.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to open the log file whose name is given
// @@ as a parameter.
// @@ Notice that as soon as the log file was opened, the opening date
// @@ and time is logged to this log file automatically.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
bool Clog::open_log (string _filename, Clog * _error_log_ptr, string _credential) {
	bool bret = true;

	if (fp != NULL) { // file already opened
		bret = false;
	}
	else {
		filename = _filename;
		fp = fopen (_filename.c_str(), "wb");

		if (!fp) {
			bret = false;
		}
		else { // log opened ok
			update_time (true);
			string stmp = "Log file creation date : ";
			stmp = stmp + tab;
			//log_that (ADMIN_LEVEL, stmp.c_str());

			credential = _credential;
			// the error log
			error_log_ptr = _error_log_ptr;
		}
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	update_time	
// @@
// @@ INPUT			:
// @@ _large_mode: bool: true if we want a large display of current date and time
// @@                    false otherwise.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	private
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to obtain the current date and time
// @@ formatted to either a condensed string, or to a developped
// @@ string.
// @@
// @@ CONTRACT 		:	none 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::update_time (bool _large_mode) {
	#if defined (__WIN32_OS__)
		GetLocalTime (&tim);
		if (_large_mode == false) {
			snprintf (tab, MAX_PATH, "%02d/%02d-%02dh%02dm%02ds ", tim.wMonth, tim.wDay, tim.wHour, tim.wMinute, tim.wSecond);
		}
		else {
			snprintf (tab, MAX_PATH, "month %d the %dth at %d hour %d minutes and %d seconds", tim.wMonth, tim.wDay, tim.wHour, tim.wMinute, tim.wSecond);
		}
	#endif // #if defined (__WIN32_OS__)
	#if defined (__LINUX_OS__)
		current_time = time (NULL);
		tm_ptr = localtime (&current_time);
		if (_large_mode == false) {
			snprintf (tab, MAX_PATH, "%02d/%02d-%02dh%02dm%02ds ", tm_ptr->tm_mon, tm_ptr->tm_mday, tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec);
		}
		else {
			snprintf (tab, MAX_PATH, "month %d the %dth at %d hour %d minutes and %d seconds", tm_ptr->tm_mon, tm_ptr->tm_mday, tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec);
		}
	#endif // #if defined (__LINUX_OS__)
	tab[MAX_PATH - 1] = '\0';
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	insert_date		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none	
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	private
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to update the current time and to
// @@ insert it into the header of a new line of the log file.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::insert_date () {
    if (fp) {
        update_time ();
		one_line.insert_date (fp, tab);
    }
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	insert_level	
// @@
// @@ INPUT			:
// @@ _level: UI32: level to be displayed.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	private
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to insert the level given as a parameter
// @@ to the header of a new line of the log file.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::insert_level (UI32 _level) {
    string level = "|???| ";
    switch (_level) {
        case NORMAL_LEVEL:
            level = "|NOR| ";
            break;
        case WARNING_LEVEL:
            level = "|WRN| ";
            break;
        case ERROR_LEVEL:
            level = "|ERR| ";
            break;
        case ADMIN_LEVEL:
            level = "|ADM| ";
            break;
        case DEBUG_LEVEL:
            level = "|DBG| ";
            break;
        case NONE_LEVEL:
            level = "     ";
            break;
    }
    if (fp) {
		one_line.insert_level (fp, level.c_str());
    }
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS				: Clog
// @@
// @@ FUNCTION			: close_log
// @@
// @@ INPUT				: none
// @@
// @@ OUTPUT			: none
// @@
// @@ IO				: none
// @@
// @@ RETURN VALUE		: none
// @@	
// @@ DISCLOSURE		: public	
// @@
// @@ DESCRIPTION:
// @@ This function just flushes the data and close the previously
// @@ opened log file.
// @@
// @@ CONTRACT			: none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::close_log() {
	if ((log_in_file == true) && (fp)) {
		fclose (fp);
		fp = NULL;
	}
	reset ();
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS				: Clog
// @@
// @@ FUNCTION			: reset
// @@
// @@ INPUT				: none
// @@
// @@ OUTPUT			: none
// @@
// @@ IO				: none
// @@
// @@ RETURN VALUE		: none
// @@	
// @@ DISCLOSURE		: public	
// @@
// @@ DESCRIPTION:
// @@ This function permits to set all fields of the
// @@ object to an initial state.
// @@
// @@ CONTRACT			: none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::reset () {
	filename		= "";
	credential		= "";
	fp				= NULL;
	error_log_ptr	= NULL;
	current_line	= 1;
	log_in_file		= true;
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that		
// @@
// @@ INPUT			:
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: char *: the message to log
// @@ _no_eol: bool: true if an end of line should be appended at the end of the
// @@                message, false otherwise.
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none	
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			one_line.insert_body (fp, _cptr);
			check_termination ();
		}
	}
#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str());	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	get_filename		
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none		
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to retrieve the name of the file
// @@ that is currently opened.
// @@
// @@ CONTRACT 		:	none 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string Clog::get_filename () {
	return (filename);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that		
// @@
// @@ INPUT			:
// @@ _level: UI32  :	the severity level associated with the message to log.
// @@ _cptr: char * :	the first message to log
// @@ _cptr2: char *:	the second message to log
// @@
// @@ OUTPUT		:	none		
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none 
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr, CCPTR _cptr2) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			string stmp = _cptr;
			stmp = stmp + _cptr2;
			one_line.insert_body (fp, stmp.c_str());
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str(), _cptr2);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that		
// @@
// @@ INPUT			:
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: char * : the first message to log
// @@ _cptr2: char *: the second message to log
// @@ _cptr3: char *: the third message to log
// @@
// @@ OUTPUT		:	none	
// @@
// @@ IO			:	none	
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr, CCPTR _cptr2, CCPTR _cptr3) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			string stmp = _cptr;
			stmp = stmp + _cptr2;
			stmp = stmp + _cptr3;
			one_line.insert_body (fp, stmp.c_str());
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str(), _cptr2, _cptr3);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that	
// @@
// @@ INPUT			:
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: char * : the first message to log
// @@ _cptr2: char *: the second message to log
// @@ _cptr3: char *: the third message to log
// @@ _cptr4: char *: the fourth message to log
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr, CCPTR _cptr2, CCPTR _cptr3, CCPTR _cptr4) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			string stmp = _cptr;
			stmp = stmp + _cptr2;
			stmp = stmp + _cptr3;
			stmp = stmp + _cptr4;
			one_line.insert_body (fp, stmp.c_str());
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str(), _cptr2, _cptr3, _cptr4);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: char *: the message to write to log file.
// @@ _val: SI32: the value to log after the message.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr, SI32 _val) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			snprintf (tmp, MAX_PATH, "%s %d", _cptr, _val);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str(), _val);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: char *: the message to write to log file.
// @@ _val: UI32: the value to log after the message.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr, UI32 _val) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			snprintf (tmp, MAX_PATH, "%s %u", _cptr, _val);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str(), _val);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _val_zero: UI32: the first value to log.
// @@ _val_un  : UI32: the second value to log.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, UI32 _val_zero, UI32 _val_un) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			snprintf (tmp, MAX_PATH, "%u %u", _val_zero, _val_un);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		char tmp[MAX_PATH];
		snprintf (tmp, MAX_PATH, "%s%u %u", credential.c_str(), _val_zero, _val_un);
		tmp[MAX_PATH - 1] = '\0';
		error_log_ptr->log_that (_level, tmp);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: CCPTR: the message to log to file.
// @@ _val: unsigned long: the value to log to file after the message.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that (UI32 _level, CCPTR _cptr, unsigned long _val) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			snprintf (tmp, MAX_PATH, "%s %u", _cptr, (UI32) _val);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _cptr;
		error_log_ptr->log_that (_level, total.c_str(), _val);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_that_hex	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _cptr: CCPTR: the message to log to file.
// @@ _value: UI32: the value to log to file after the message.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function logs the message(s) given as parameter(s) and associate
// @@ with them the current time and a severity level given as a parameter.
// @@ Notice that the '_value' parameter os logged in hexadecimal format.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_that_hex (UI32 _level, CCPTR _msg, UI32 _value) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			snprintf (tmp, MAX_PATH, "%s %X", _msg, _value);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();
		}
	}

#if defined (__REPORT_TO_ERROR_LOG__)
	if ((error_log_ptr != NULL) && (_level == ERROR_LEVEL)) {
		string total = credential + _msg;
		error_log_ptr->log_that_hex (_level, total.c_str(), _value);	
	}
#endif // #if defined (__REPORT_TO_ERROR_LOG__)
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_buffer	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _buffer: UI08 *: the address of the first byte of the buffer to log.
// @@ _size: UI32: the size of the buffer to log.
// @@ _comment: CCPTR: the comment associated with the buffer to log.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function is for debug purpose only.
// @@ it dump the content of the buffer '_buffer' into the 
// @@ file : C:\\binary.bin, the format of the file being
// @@ a readable sequence of hexadecimal values (a block).
// @@ So it can be seen as an outbound save up function.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_buffer (UI32 _level, UI08 * _buffer, UI32 _size, CCPTR _comment) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			char tmp2[MAX_PATH];
			snprintf (tmp, MAX_PATH, "=== BUFFER OF : %s ===", _comment);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();

			tmp[0]			= '\0';
			UI32 nb_byte	= 0;
			UI32 i			= 0;
			FILE * fpzx		= NULL;

			fpzx = fopen ("C:\\binary.bin", "wb");

			if (! fpzx) {
#if defined (__WIN32_OS__) 
				MessageBox (NULL, "failed to open bin log file !", "", MB_ICONERROR);
#endif // #if defined (__WIN32_OS__) 
			}

			for (i=0 ; i<_size ; i++) {
				fwrite ((char *) &_buffer[i], 1, 1, fpzx);
				fflush (fpzx);
				snprintf (tmp2, MAX_PATH, "%02X ", _buffer[i]);
				tmp2[MAX_PATH - 1] = '\0';
				strcat (tmp, tmp2);
				nb_byte++;

				if (nb_byte >= 16) {
					insert_date ();
					insert_level (_level);
					one_line.insert_body (fp, tmp);
					tmp[0] = '\0'; // reset the cumulative line
					nb_byte = 0;
				}
			}

			fclose (fpzx);

			// flush the remaining bytes (if any)
			if (nb_byte > 0) {
				insert_date ();
				insert_level (_level);
				one_line.insert_body (fp, tmp);
			}
		}
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	log_buffer	
// @@
// @@ INPUT			:	
// @@ _level: UI32: the severity level associated with the message to log.
// @@ _buffer: UI08 *: the address of the first byte of the buffer to log.
// @@ _comment: CCPTR: the comment associated with the buffer to log.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function is for debug purpose only.
// @@ it permits to dump to the log file the content of 
// @@ a given buffer containing the opcode of a given function.
// @@ Notice that this function stops to dump when it reaches
// @@ a RET (0x3C) or a NOP (0x90) instruction.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::log_code_buffer (UI32 _level, UI08 * _buffer, CCPTR _comment) {
	if (log_in_file == true) {
		if (fp) {
			insert_date ();
			insert_level (_level);
			char tmp[MAX_PATH];
			char tmp2[MAX_PATH];
			snprintf (tmp, MAX_PATH, "=== %s ===", _comment);
			tmp[MAX_PATH - 1] = '\0';
			one_line.insert_body (fp, tmp);
			check_termination ();

			tmp[0]			= '\0';
			UI32 nb_byte	= 0;
			UI32 i			= 0;

			for (i=0 ; ; i++) {
				snprintf (tmp2, MAX_PATH, "%02X ", _buffer[i]);
				tmp2[MAX_PATH - 1] = '\0';
				strcat (tmp, tmp2);
				nb_byte++;

				if (nb_byte >= 32) {
					insert_date ();
					insert_level (_level);
					one_line.insert_body (fp, tmp);
					tmp[0] = '\0'; // reset the cumulative line
					nb_byte = 0;
				}

				// functions finish by a RET + NOP instruction
				if ((i>256) || ((i > 0) && (_buffer[i] == 0x90) && (_buffer[i-1] == 0x3C))) { 
					break;
				}

			}

			// flush the remaining bytes (if any)
			if (nb_byte > 0) {
				insert_date ();
				insert_level (_level);
				one_line.insert_body (fp, tmp);
			}
		}
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	get_file_descriptor	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ FILE * : the file descriptor of the opened log file.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to retrieve the descriptor of
// @@ the log file (so that external functions can use this
// @@ descriptor to do their job if necessary).
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
FILE * Clog::get_file_descriptor () {
	return (fp);	
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	Clog
// @@
// @@ FUNCTION		:	force_flush	
// @@
// @@ INPUT			:	none	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to ensure that all the data
// @@ are flushed to the log file by closing and reopening this
// @@ file. Notice That the performance decrease if you use this
// @@ function.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void Clog::force_flush () {
	fflush (fp);	
}


void display_result (Clog * _log_ptr, CCPTR _fn_name, UI32 _success_iret, UI32 _actual_iret) {
	if (_log_ptr != NULL) {
		char tmp[MAX_PATH];
		if (_actual_iret == _success_iret) {
			snprintf (tmp, MAX_PATH, "kernel_manager - function '%s'...[SUCCESS]", _fn_name);
			tmp[MAX_PATH - 1] = '\0';
			_log_ptr->log_that (NORMAL_LEVEL, tmp);	
		}
		else {
			string serror = err_2_string (_actual_iret);
			snprintf (tmp, MAX_PATH, "+++ ERROR: function '%s' failed ! (%s)", _fn_name, serror.c_str());
			tmp[MAX_PATH - 1] = '\0';
			_log_ptr->log_that (ERROR_LEVEL, tmp);	
		}
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
