// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: log.hpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains the definition of the 
// @@ log facility that permits to log events and
// @@ to associate a time and date, coming along
// @@ with a severity level for each event.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ multiple inclusion problem avoidance
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#ifndef __MIGALE_LOG__
#define __MIGALE_LOG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include "../lib/general.hpp"
#if defined (__WIN32_OS__) 
//	#include <Windows.h>
	#include <windows.h>
//	#include <winnt.h>
#endif // #if defined (__WIN32_OS__) 
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <string>
#include <iostream>
#include <map>



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ namespace usage declaration
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
using namespace std;


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ log related constants
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define LOG_MAX_NB_LINE				65536
#define LOG_TMP_BUF_MAX_SIZE		256

//"02/20-17h51m48s "
#define DATE_LENGTH					16
//"|NOR| "
#define LEVEL_LENGTH				6

#if defined (__WIN32_OS__) 
	#define LINE_LENGTH				512 - DATE_LENGTH - LEVEL_LENGTH
#endif // #if defined (__WIN32_OS__) 
#if defined (__LINUX_OS__)
	#define LINE_LENGTH				256 - DATE_LENGTH - LEVEL_LENGTH
#endif // #if defined (__LINUX_OS__)



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ log related constants
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define LOG_MAX_NB_LINE				65536
#define LOG_TMP_BUF_MAX_SIZE		256

//"02/20-17h51m48s "
#define DATE_LENGTH					16
//"|NOR| "
#define LEVEL_LENGTH				6

#if defined (__WIN32_OS__) 
	#define LINE_LENGTH				512 - DATE_LENGTH - LEVEL_LENGTH
#endif // #if defined (__WIN32_OS__) 
#if defined (__LINUX_OS__)
	#define LINE_LENGTH				256 - DATE_LENGTH - LEVEL_LENGTH
#endif // #if defined (__LINUX_OS__)

#if defined (__WIN32_OS__) 
	#define snprintf _snprintf
#endif // #if ! defined (__WIN32_OS__) 



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ log relative constants
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
enum possible_log_levels_enum {
    NORMAL_LEVEL,
    WARNING_LEVEL,
    ERROR_LEVEL,
    ADMIN_LEVEL,
    DEBUG_LEVEL,
    NONE_LEVEL,
};

#if defined (__LINUX_OS__)
	#define MAX_PATH	256
#endif // #if defined (__LINUX_OS__)




// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS: Cone_line
// @@
// @@ DESCRIPTION:
// @@ This object permits to have a fixed size line
// @@ composed of 
// @@ - a date
// @@ - a severity level
// @@ - a message
// @@
// @@ Notice that if the message to append to the line
// @@ exceeds the line's maximum size, then the remaining
// @@ part of the message is lost.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
class Cone_line
{
private:
	string terminator;
	void insert_terminator (FILE *);

public:

	Cone_line () {
#if defined (__WIN32_OS__)
		terminator = "\r\n";
#endif // #if defined (__WIN32_OS__)
#if defined (__LINUX_OS__)
		terminator = "\n";
#endif // #if defined (__LINUX_OS__)
	}

	void insert_date (FILE *, CCPTR);
	void insert_level (FILE *, CCPTR);
	void insert_body (FILE *, CCPTR);
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS: Clog
// @@
// @@ DESCRIPTION:
// @@ This object permits to open a log file,
// @@ save data into it, and close it.
// @@ It can be thread safe depending on the compilation
// @@ flags you choose.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
class Clog
{
	private:
		string		filename;
		string		credential;
		Cone_line	one_line;
		FILE *		fp;
		Clog *		error_log_ptr;
		UI32		current_line;
		bool		log_in_file;
		char		tab[LOG_TMP_BUF_MAX_SIZE];
#if defined (__WIN32_OS__)
		SYSTEMTIME	tim;
#endif // #if defined (__WIN32_OS__)
#if defined (__LINUX_OS__)
        time_t      current_time;
        struct tm * tm_ptr;
#endif // #if defined (__LINUX_OS__)
		void		update_time (bool _large_mode=false);
		void		insert_date ();
		void		insert_level (UI32);
		void		check_termination ();
		void		reset ();

	public:
		Clog();
		virtual		~Clog();
		bool		open_log (string, Clog *, string);
		void		close_log ();
		void		log_that (UI32, CCPTR);
		void		log_that (UI32, CCPTR, SI32);
		void		log_that (UI32, CCPTR, UI32);
		void		log_that (UI08);
		void		log_that (UI32, UI32, UI32);
		void		log_that (UI32, CCPTR, CCPTR);
		void		log_that (UI32, CCPTR, CCPTR, CCPTR);
		void		log_that (UI32, CCPTR, CCPTR, CCPTR, CCPTR);
		void		log_that (UI32, CCPTR, unsigned long);
		void		log_that (UI32, UI32);
		void		log_that_hex (UI32, CCPTR, UI32);
		void		log_code_buffer (UI32, UI08 *, CCPTR);
		void		log_buffer (UI32, UI08 *, UI32, CCPTR);
		FILE *		get_file_descriptor ();
		string		get_filename ();
		void		force_flush ();
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ misc functions declarations
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void display_file_2_log (CCPTR, Clog&); 
string err_2_string (SI32);
void display_result (Clog *, CCPTR, UI32, UI32);



#endif // __MIGALE_LOG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
