// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: inthepicture.cpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains all functions and
// @@ data structures that are necessary for the
// @@ plugin 'inThePicture' to perform it's task.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#include "../lib_plugin/general_plugin.hpp"



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	reset_log				
// @@
// @@ INPUT			:	none
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to clear the plugin debug log file
// @@ by reseting it's content.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void reset_log () {
	// reset log file
	FILE * fp_log = fopen ("C:\\inthepicture_log.txt", "wb");
	if (fp_log) {
		fclose (fp_log);
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	log_that				
// @@
// @@ INPUT			:	
// @@ _msg: CCPTR: the text message to save into the log file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to save the '_msg' ascii string
// @@ into the debug file whose location and name are:
// @@ "C:\\inthepicture_log.txt"
// @@ Notice that the log file is opened and closed everytime 
// @@ this function is called so that debug is possible even
// @@ when a crash occurs.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void log_that (CCPTR _msg) {
	FILE * fp_log = fopen ("C:\\inthepicture_log.txt", "ab");
	if (fp_log) {
		fprintf (fp_log, "%s\r\n", _msg);
		fclose (fp_log);
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	log_vector				
// @@
// @@ INPUT			:	
// @@ _msg: CCPTR: the text message to save into the log file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	
// @@ _v: vector<UI08> : the vector to save to log file.
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to save the content of the
// @@ byte vector given by the parameter '_v'
// @@ into the debug file whose location and name are:
// @@ "C:\\inthepicture_log.txt"
// @@
// @@ Notice that the log file is opened and closed everytime 
// @@ this function is called so that debug is possible even
// @@ when a crash occurs.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void log_vector (CCPTR _msg, vector<UI08>& _v) {
	FILE * fp_log = fopen ("C:\\inthepicture_log.txt", "ab");
	if (fp_log) {
		UI32 size, i;
		size = _v.size ();
		fprintf (fp_log, "%s: [", _msg);
		for (i=0 ; i<size ; i++) {
			fprintf (fp_log, "%02X ", _v[i]);	
		}
		fprintf (fp_log, "]\r\n");
		fclose (fp_log);
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	log_vector				
// @@
// @@ INPUT			:	
// @@ _msg: CCPTR: the text message to save into the log file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	
// @@ _v_extracted: vector<UI08> : the first vector to save to log file.
// @@ _v_remaining: vector<UI08> : the second vector to save to log file.
// @@	
// @@ RETURN VALUE	:	none
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to save the content of the
// @@ byte vector given by the parameter '_v_extracted'
// @@ and '_v_remaining' in the file whose location and name are:
// @@ "C:\\inthepicture_log.txt"
// @@
// @@ Notice that the log file is opened and closed everytime 
// @@ this function is called so that debug is possible even
// @@ when a crash occurs.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void log_vectors (CCPTR _msg, vector<UI08>& _v_extracted, vector<UI08>& _v_remaining) {
	FILE * fp_log = fopen ("C:\\inthepicture_log.txt", "ab");
	if (fp_log) {
		UI32 size, i;
		fprintf (fp_log, "%s: v_extracted=[", _msg);
		size = _v_extracted.size ();
		for (i=0 ; i<size ; i++) {
			fprintf (fp_log, "%02X ", _v_extracted[i]);
		}
		fprintf (fp_log, "] , v_remaining=[");
		size = _v_remaining.size ();
		for (i=0 ; i<size ; i++) {
			fprintf (fp_log, "%02X ", _v_remaining[i]);	
		}
		fprintf (fp_log, "]\r\n");
		fclose (fp_log);
	}
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	get_file_nb_line			
// @@
// @@ INPUT			:
// @@ _filename: name of the text file whose number of lines is to
// @@            be computed.
// @@
// @@ OUTPUT		:
// @@ _nb_line: UI32: number of lines contained in the file whose
// @@       name is given by the '_filename' parameter.	
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ bool: true if operation was succesful, 
// @@       false otherwise.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to compute the number of lines 
// @@ of the file whose name is given as the '_filename' parameter,
// @@ the result being stored into the '_nb_line' argument.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@	
bool get_file_nb_line (CCPTR _filename, UI32& _nb_line) {
	bool bret	= true;
	_nb_line	= 0;
	FILE * fp = fopen (_filename, "rb");
	if (! fp) {
		bret = false;
	}
	else {
		char buf[1024];
		char * cptr = fgets (buf, 1024, fp);
		while (cptr != NULL) {
			_nb_line++;
			cptr = fgets (buf, 1024, fp);
		}
		fclose (fp);
	}
	return (bret);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	get_bmp_depth			
// @@
// @@ INPUT			:	
// @@ _ptr: UI08 *: address of the first byte of the mmapped file
// @@               to analyze.
// @@
// @@ _size: UI32: size of the mmapped file.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ UI32: the depth of the BMP file being under analysis.
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to retrieve the color depth of the
// @@ BMP file that was previously mmapped, the first byte of the
// @@ mmapped file being stored into the '_ptr' parameter, and the
// @@ size of the mmapped file being stored into the '_size' 
// @@ parameter.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
UI32 get_bmp_depth (UI08 * _ptr, UI32 _size) {
	UI32 depth = 0;
	union Atype_tmp {
		unsigned short ashort;
		unsigned char tab[2];
	};

	Atype_tmp atyp;
	atyp.tab[0] = _ptr[28];
	atyp.tab[1] = _ptr[29];
	depth = atyp.ashort;
	return(depth);
}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	extract_filename_from_path			
// @@
// @@ INPUT			:
// @@ _path: CCPTR: the full filename (absolute).	
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the filename whose folder has been removed.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function permits to extract the filename from an
// @@ absolute filename composed of both folder and filename.
// @@ Thus this function permits to remove the folder's name.
// @@
// @@ CONTRACT 		:	
// @@ - '_path' parameter must not be an empty string.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string extract_filename_from_path (CCPTR _path) {
	string filename;
	UI32 size = strlen (_path);
	if (size > 0) {
		UI32 start = size - 1;
		while ((start > 0) && (_path[start] != '/') && (_path[start] != '\\')) {
			start--;
		}

		for (UI32 i=start+1 ; i<size ; i++) {
			filename += _path[i];	
		}
	}
	return (filename);
}


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@
// @@ CLASS			:	-	
// @@
// @@ FUNCTION		:	plugin_err_2_string			
// @@
// @@ INPUT			:
// @@ _ercode: UI32: the error code to convert.
// @@
// @@ OUTPUT		:	none
// @@
// @@ IO			:	none
// @@	
// @@ RETURN VALUE	:	
// @@ string: the error code converted into a 
// @@         human-readable string.	
// @@	
// @@ DISCLOSURE	:	public
// @@	
// @@ DESCRIPTION	:
// @@ This function converts the '_ercode' parameter into
// @@ a human-readable string. This parameter being the
// @@ numeric value that uniquely identifies a given return value.
// @@
// @@ CONTRACT 		:	none
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
string plugin_err_2_string (SI32 _ercode) {
	string s;
	switch (_ercode) {
		// === plugins related error codes === 
		case PLUGIN_SUCCESS:					s = "PLUGIN_SUCCESS";					break;
		case PLUGIN_LIBRARY_OPEN_FAILED:		s = "PLUGIN_LIBRARY_OPEN_FAILED";		break;
		case PLUGIN_ILLEGAL_WHEN_LOADED:		s = "PLUGIN_ILLEGAL_WHEN_LOADED";		break;
		case PLUGIN_FN_LOOKUP_FAILED:			s = "PLUGIN_FN_LOOKUP_FAILED";			break;
		case PLUGIN_NO_SUCH_FILE:				s = "PLUGIN_NO_SUCH_FILE";				break;
		case PLUGIN_FREE_LIB_FAILED:			s = "PLUGIN_FREE_LIB_FAILED";			break;
		case PLUGIN_XXXCIPHER_MMAP_FAILED:		s = "PLUGIN_XXXCIPHER_MMAP_FAILED";		break;
		case PLUGIN_EXTRACT_PARSE_ERROR:		s = "PLUGIN_EXTRACT_PARSE_ERROR";		break;
		case PLUGIN_FILENAME_EXTRACT_FAILED:	s = "PLUGIN_FILENAME_EXTRACT_FAILED";	break;
		case PLUGIN_FILE_FOPEN_FAILED:			s = "PLUGIN_FILE_FOPEN_FAILED";			break;
		case PLUGIN_FILE_FCLOSE_FAILED:			s = "PLUGIN_FILE_FCLOSE_FAILED";		break;
		case PLUGIN_FILE_CONTENT_EXTRACT_ERROR:	s = "PLUGIN_FILE_CONTENT_EXTRACT_ERROR";break;
		case PLUGIN_FWRITE_FAILED:				s = "PLUGIN_FWRITE_FAILED";				break;
		case PLUGIN_FILE_HEADER_EXTRACT_ERROR:	s = "PLUGIN_FILE_HEADER_EXTRACT_ERROR";	break;
		case PLUGIN_INVALID_PARAMETER:			s = "PLUGIN_INVALID_PARAMETER";			break;
		case PLUGIN_END_OF_MEDIA_REACHED:		s = "PLUGIN_END_OF_MEDIA_REACHED";		break;
		case PLUGIN_HIDDEN_DATA_SIZE_EXTRACT_FAILED:	s = "PLUGIN_HIDDEN_DATA_SIZE_EXTRACT_FAILED";	break;
		case PLUGIN_END_OF_FILE_HEADER_REACHED:			s = "PLUGIN_END_OF_FILE_HEADER_REACHED"; break;

		// ==== analysis result related errors =====
		case ANALYSIS_RESULT_SUCCESS:			s = "ANALYSIS_RESULT_SUCCESS";			break;
		case ANALYSIS_RESULT_FOPEN_FAILED:		s = "ANALYSIS_RESULT_FOPEN_FAILED";		break;
		case ANALYSIS_RESULT_GET_NB_LINE_FAILED:s = "ANALYSIS_RESULT_GET_NB_LINE_FAILED";break;
		case ANALYSIS_RESULT_PARSE_ERROR:		s = "ANALYSIS_RESULT_PARSE_ERROR";		break;
		case ANALYSIS_RESULT_VAR_MISMATCH:		s = "ANALYSIS_RESULT_VAR_MISMATCH";		break;

		default: s = "<unknown_error>";		
	}
	return (s);
}


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
