// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ FILE: general_plugin.hpp
// @@ 
// @@ DESCRIPTION:
// @@ This file contains all functions and
// @@ data structures that are useful for any
// @@ kind of plugin.
// @@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ multiple inclusion problem avoidance
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#ifndef __EXOSTEG_PLUGIN_GENERAL_FLAG__
#define __EXOSTEG_PLUGIN_GENERAL_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ includes
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//#include "../../lib/general.hpp"
//#include "../../lib/hidden_data.hpp"
#include <string>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <string>
#include "../lib_plugin/data_types.hpp"
#include <windows.h>



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ namespace usage declaration
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
using namespace std;



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ constants
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
const UI32 BMP_HEADER_SIZE = 54;



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ possible files types list
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
enum file_type_enum {
	FILE_TYPE_NOT_KNOWN = 0,
	FILE_TYPE_JPG,
	FILE_TYPE_GIF,
	FILE_TYPE_BMP,
	FILE_TYPE_MPEG,
	FILE_TYPE_MP3,
	FILE_TYPE_WMV,
	FILE_TYPE_TXT,
	FILE_TYPE_TXT_UTF8,
	FILE_TYPE_TXT_UNICODE,
	FILE_TYPE_TXT_UNICODE_BIG_ENDIAN,
	FILE_TYPE_EXE,
	FILE_TYPE_ZIP,
	FILE_TYPE_AVI,
	FILE_TYPE_ASF,
	FILE_TYPE_KROSOFT,
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ possible operations result 
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
enum possible_operation_result_flag_enum {
	OPA_NOT_DONE = 0,
	OPA_DONE,
	OPA_FAILED,
	OPA_NOT_FOUND,
	OPA_USED,
	OPA_NOT_USED,
	OPA_NONE,
	OPA_YES,
	OPA_NO,
};


// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ return codes for the entire application
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
enum antisteg_plugins_return_codes_enum {
	// === plugins related error codes ===
	PLUGIN_SUCCESS = 74000,	
	PLUGIN_LIBRARY_OPEN_FAILED,
	PLUGIN_ILLEGAL_WHEN_LOADED,
	PLUGIN_FN_LOOKUP_FAILED,
	PLUGIN_NO_SUCH_FILE,
	PLUGIN_FREE_LIB_FAILED,
	PLUGIN_EXTRACTION_FAILED,
	PLUGIN_XXXCIPHER_MMAP_FAILED,
	PLUGIN_CIPHER_INCOHERENCY,
	PLUGIN_UNCIPHER_INCOHERENCY,
	PLUGIN_EXTRACT_PARSE_ERROR,
	PLUGIN_FILENAME_EXTRACT_FAILED,
	PLUGIN_FILE_FOPEN_FAILED,
	PLUGIN_FILE_FCLOSE_FAILED,
	PLUGIN_FILE_CONTENT_EXTRACT_ERROR,
	PLUGIN_FWRITE_FAILED,
	PLUGIN_FILE_HEADER_EXTRACT_ERROR,
	PLUGIN_INVALID_PARAMETER,
	PLUGIN_END_OF_MEDIA_REACHED,
	PLUGIN_HIDDEN_DATA_SIZE_EXTRACT_FAILED,
	PLUGIN_END_OF_FILE_HEADER_REACHED,

	// ==== analysis result related errors =====
	ANALYSIS_RESULT_SUCCESS = 75000,
	ANALYSIS_RESULT_FOPEN_FAILED,
	ANALYSIS_RESULT_GET_NB_LINE_FAILED,
	ANALYSIS_RESULT_PARSE_ERROR,
	ANALYSIS_RESULT_VAR_MISMATCH,
};



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@ general functions used by plugins
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
void	reset_log					();
void	log_that					(CCPTR);
void	log_vector					(CCPTR, vector<UI08>&);
void	log_vectors					(CCPTR, vector<UI08>&, vector<UI08>&);
bool	get_file_nb_line			(CCPTR, UI32&);
UI32	get_bmp_depth				(UI08 *, UI32);
string	extract_filename_from_path	(CCPTR);
string	plugin_err_2_string			(SI32);


#endif // __EXOSTEG_PLUGIN_GENERAL_FLAG__



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@  end of file
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
